/*
/***********************r************************************************************
****            Procedure para Preparacao do Orcamento Operacional             ******
*************************************************************************************

Criado em  : 01/2009
       Por : Alessandra Cristina

Descricao:

          - Gera os registro para elaborao do oramento da filial, tabela ORCAMENTOOP e
          a lista de aprovacao do oramento. Pode ser executada para uma lista de operadores e
          uma lista de filiais relacionadas aos operadores, ou ainda para um unico operador

Util :
       - Dir      : @C:\WEB_509\SQL\END_CONV\ORACLE\4500_GOPSTPO_PREPARA_ORCAMENTO.SQL

       - Erro     : raise_application_error(-20501, 'Mensagem', True);

       }

*/

CREATE OR REPLACE PROCEDURE PREPARA_ORCAMENTO(
   P_NRANOORCAOP    IN ORCAMENTOOP.NRANOORCAOP%TYPE,
   P_CDVERSORCAOP   IN ORCAMENTOOP.CDVERSORCAOP%TYPE,
   P_CDOPERADOR     IN VARCHAR,
   P_CDFILIAL       IN VARCHAR DEFAULT NULL,
   P_IDEXCLUI       IN VARCHAR DEFAULT 'N',
   P_LISTAOPER      IN VARCHAR DEFAULT 'N',
   P_LISTAFIL       IN VARCHAR DEFAULT 'N',
   P_ORCADETA       IN VARCHAR DEFAULT 'N',
   P_ITEMORCA       IN VARCHAR DEFAULT NULL,
   P_CDOPERLOGADO   IN VARCHAR DEFAULT NULL)
AS
   V_NRITEMORCA      VARCHAR2(10);
   V_FILIALPAR       VARCHAR2(4);
   V_CDFILCENTPME    VARCHAR2(4);
   V_IDFATUROP       VARCHAR2(2);
   V_IDSERVFATROP    CHAR;
   V_IDFILIALAGRROP  CHAR;
   V_IDUTPADRROPFIL  CHAR;
   V_IDUTPADRROP     CHAR;
   V_EXIST_MOFIL     INTEGER;
   V_EXIST_ORCFIL    INTEGER;
   V_EXIST_MOGER     INTEGER;
   V_TET_FILIMP      VARCHAR2(4);
   FIL_JA_USADA      VARCHAR2(4);
   V_ORCDESPENTROP   VARCHAR2(1);
   V_FILIAL          VARCHAR2(4);
   V_EXISTE          VARCHAR2(1);
   V_IDSTATUSORCA    VARCHAR2(1);
   V_NRITEMORCAGOP   NUMBER;
   V_NRORCAMENTOGOP  NUMBER;
   V_IDAPROVACAOGOP  VARCHAR2(30);
   V_IDPERIODOORCA   VARCHAR2(30);
   V_NRITORCAMESGOP  NUMBER;
   V_IDVERIORCFILI   VARCHAR2(30);
   V_CDSERVFILI      VARCHAR2(05);
   C_ORCAMENTO_CDFILIAL VARCHAR2(04);
   V_CDTIPOOPER      VARCHAR2(02);
   V_CDIMPOSTOREC    VARCHAR2(02);
   V_CDIMPOSTOSOB    VARCHAR2(02);
   V_STATUSCONC      VARCHAR2(30);
   V_MENSAGEMLOG     VARCHAR2(100);
   V_NRLOGPREPARAORCAGOP NUMBER;
   V_IDUTILPERCFILGOP VARCHAR2(1);
   
    /* Variveis para LOG*/
   V_NMTABELA      VARCHAR2(20);
   V_NMOPERADOR    VARCHAR2(40);
   V_TIPO          VARCHAR2(1);
   V_DESC          VARCHAR2(500);
   V_ITPESQ        VARCHAR2(40);
   V_OPERADOR      VARCHAR2(12);
   V_EXISTEORCA    VARCHAR2(1);
   V_UNIDADE       VARCHAR2(50);
   V_NMFILIAL      VARCHAR2(40);
   V_DSSERVERRDM   VARCHAR2(50);
   V_ITENS         VARCHAR2(2000);
   V_NRANOORCAGOP  VARCHAR2(4);
   V_NRPERIODOORCA VARCHAR2(13);

   V_IDEXCLUI      VARCHAR2(01);
   V_OUTRPERIODO   VARCHAR2(30);

   /* C_SERVICO foi inativada na conversão para não detalhar por serviço. Futuramente
      essa rotina pode ser implementada.
   */
   CURSOR C_SERVICOM IS SELECT SC.CDSERVFILI
                          FROM SERVICOROP SC, SERVICO S
                         WHERE SC.CDFILIAL = TIPOPARAM_DRO(C_ORCAMENTO_CDFILIAL)
                           AND IDTPSERVICOROP = 'E'
                           AND V_IDSERVFATROP <> 'S'
                           AND SC.CDSERVFILI = S.CDSERVFILI
                           AND S.CDFILIAL = C_ORCAMENTO_CDFILIAL
                           AND 'DETALHASERVICO' = 'NAO'
                         UNION
                        SELECT CDSERVFILI
                          FROM SERVICO S
                         WHERE S.CDFILIAL = C_ORCAMENTO_CDFILIAL
                           AND V_IDSERVFATROP = 'S'
                           AND (QTCOMESTISEG IS NOT NULL OR
                                QTCOMESTITER IS NOT NULL OR
                                QTCOMESTIQUA IS NOT NULL OR
                                QTCOMESTIQUI IS NOT NULL OR
                                QTCOMESTISEX IS NOT NULL OR
                                QTCOMESTISAB IS NOT NULL OR
                                QTCOMESTIDOM IS NOT NULL OR
                                QTCOMESTIFER IS NOT NULL)
                           AND 'DETALHASERVICO' = 'NAO';

   CURSOR C_DESPENTRADA IS SELECT CDTIPOOPER
                             FROM TIPOOPERACROP
                            WHERE IDTPOPERACROP = 'DE'
                              AND IDENTRSAIDOP = 'E'
                              AND CDFILIAL = V_FILIALPAR
                            MINUS
                           SELECT CDTIPOOPER
                             FROM ITEMORCAGOP
                            WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                              AND CDFILIAL = C_ORCAMENTO_CDFILIAL
                              AND IDTIPOORCAOP = 'DESPENTRADA'
                              AND IDENTRSAIDOP = 'E';

   CURSOR C_IMPOSTOARECUPERAR IS SELECT PD.CDIMPOSTO
                                   FROM PERCITEMDETALGOP PD INNER JOIN PERCITEMGOP P ON PD.NRPERCITEMGOP = P.NRPERCITEMGOP
                                  WHERE P.NRSEQITEMORCA = 8  
                                    AND V_IDUTILPERCFILGOP <> 'S'
                                  UNION
                                 SELECT PDF.CDIMPOSTO
                                   FROM PERCITFILGOP PF LEFT JOIN PERCITDETALFILGOP PDF ON PDF.NRPERCITFILGOP = PF.NRPERCITFILGOP
                                  WHERE PDF.CDFILIAL = C_ORCAMENTO_CDFILIAL
                                    AND PF.NRSEQITEMORCA = 8               
                                    AND V_IDUTILPERCFILGOP = 'S'
                                  MINUS
                                 SELECT CDIMPOSTO
                                   FROM ITEMORCAGOP
                                  WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                    AND CDFILIAL = C_ORCAMENTO_CDFILIAL
                                    AND IDTIPOORCAOP = 'IMPOSTOARECUPERAR';

   CURSOR C_IMPOSTOSOBLUCRO IS SELECT PD.CDIMPOSTO
                                 FROM PERCITEMDETALGOP PD INNER JOIN PERCITEMGOP P ON PD.NRPERCITEMGOP = P.NRPERCITEMGOP
                                WHERE P.NRSEQITEMORCA = 9  
                                  AND V_IDUTILPERCFILGOP <> 'S'
                                UNION
                               SELECT PDF.CDIMPOSTO
                                 FROM PERCITFILGOP PF LEFT JOIN PERCITDETALFILGOP PDF ON PDF.NRPERCITFILGOP = PF.NRPERCITFILGOP
                                WHERE PDF.CDFILIAL = C_ORCAMENTO_CDFILIAL
                                  AND PF.NRSEQITEMORCA = 9           
                                  AND V_IDUTILPERCFILGOP = 'S'
                                MINUS
                               SELECT CDIMPOSTO
                                 FROM ITEMORCAGOP
                                WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                  AND CDFILIAL = C_ORCAMENTO_CDFILIAL
                                  AND IDTIPOORCAOP = 'IMPOSTOSOBLUCRO';

   PROCEDURE PREPARA_MES(P_CDFILIAL IN VARCHAR2, P_PERIODO IN NUMBER, P_NRITEMORCAOP IN NUMBER, P_DETALHA IN VARCHAR2,
                         P_ITEM IN NUMBER, P_CDDESPPREV IN VARCHAR2 DEFAULT NULL)
   AS
     V_NRITORCAMESGOP    NUMBER;
     V_ITDETALORCAMESGOP NUMBER;
     V_DESPESA           VARCHAR2(30);
     V_ORCAMENTO         VARCHAR2(30);
   BEGIN                        
      IF P_ITEM = 2 THEN
         V_DESPESA   := 'CONSUMO';
         V_ORCAMENTO := 'CONSUMO';
      ELSIF P_ITEM = 5 THEN
         V_DESPESA   := 'MATRIZ';
         V_ORCAMENTO := 'DESPMATRIZ';
      ELSIF P_ITEM = 6 THEN
         V_DESPESA   := 'DIVERSA';
         V_ORCAMENTO := 'DESPDIVERSA';
      ELSIF P_ITEM = 7 THEN
         V_DESPESA   := 'LANCENTRADA';
         V_ORCAMENTO := 'DESPENTRADA';
      END IF;

      FOR C_MES IN (SELECT ADD_MONTHS(TO_DATE(CASE WHEN NVL(F.NRDIAINIORCAGOP,P.NRDIAINIORCA) > TO_NUMBER(TO_CHAR(LAST_DAY(D.DTMES),'DD'))
                                                   THEN TO_CHAR(LAST_DAY(D.DTMES),'DD')
                                                   ELSE LPAD(NVL(F.NRDIAINIORCAGOP,P.NRDIAINIORCA),2,'0')
                                              END ||'/'||TO_CHAR(D.DTMES,'MM')||'/'||P_NRANOORCAOP,'DD/MM/YYYY'), NVL(P.NRMESINIORCA,'01')-1) DT
                      FROM PERIODOORCAGOP PO, FILIAL F, PARAMORCAOPERAC P, DATA D
                     WHERE F.CDFILIAL = P_CDFILIAL
                       AND PO.NRPERIODO = P_PERIODO
                       AND D.DTMES BETWEEN PO.DTINIPERIORCA AND PO.DTFIMPERIORCA
                     GROUP BY ADD_MONTHS(TO_DATE(CASE WHEN NVL(F.NRDIAINIORCAGOP,P.NRDIAINIORCA) > TO_NUMBER(TO_CHAR(LAST_DAY(D.DTMES),'DD'))
                                                      THEN TO_CHAR(LAST_DAY(D.DTMES),'DD')
                                                      ELSE LPAD(NVL(F.NRDIAINIORCAGOP,P.NRDIAINIORCA),2,'0')
                                                 END ||'/'||TO_CHAR(D.DTMES,'MM')||'/'||P_NRANOORCAOP,'DD/MM/YYYY'), NVL(P.NRMESINIORCA,'01')-1)
                     ORDER BY DT) LOOP

         SELECT MAX(NRITORCAMESGOP)
           INTO V_NRITORCAMESGOP
           FROM ITORCAMESGOP
          WHERE NRITEMORCAOP = P_NRITEMORCAOP
            AND DTORCAMENTO = C_MES.DT;

         IF V_NRITORCAMESGOP IS NULL THEN 
            NOVO_CODIGO('ITORCAMESGOP', V_NRITORCAMESGOP);

            INSERT INTO ITORCAMESGOP
                        (NRITEMORCAOP, NRITORCAMESGOP, DTORCAMENTO, IDALTMANUAL, VRORCAMENTO,
                         VRREAJUSTE, VRPERCENTUAL, QTORCAMENTO)
                 VALUES (P_NRITEMORCAOP, V_NRITORCAMESGOP, C_MES.DT, 'N', 0,
                         0, 0, 0);
         END IF;

         IF P_DETALHA = 'S' THEN
            IF P_ITEM IN (5,6,7) THEN
               FOR C_DETALHAMESFIL IN (SELECT PP.CDPRODUTO
                                         FROM PERCITFILGOP P INNER JOIN PERCITDETALFILGOP PD
                                                                     ON P.NRPERCITFILGOP = PD.NRPERCITFILGOP
                                                             INNER JOIN PERCDETPRODFILGOP PP
                                                                     ON PD.NRPERCITDETALFILGOP = PP.NRPERCITDETALFILGOP
                                        WHERE P.NRSEQITEMORCA = P_ITEM
                                          AND PD.IDDESPPREV = V_DESPESA
                                          AND P.CDFILIAL = P_CDFILIAL   
                                          AND NVL(PD.CDDESPPREV,'T') = NVL(P_CDDESPPREV,'T')
                                        MINUS
                                       SELECT IT.CDPRODUTO
                                         FROM ITEMORCAGOP I INNER JOIN ITORCAMESGOP IM ON I.NRITEMORCAOP = IM.NRITEMORCAOP
                                                            INNER JOIN ITDETALORCAMESGOP IT ON IM.NRITORCAMESGOP = IT.NRITORCAMESGOP
                                        WHERE IT.NRITORCAMESGOP = V_NRITORCAMESGOP
                                          AND I.IDTIPOORCAOP = V_ORCAMENTO) LOOP

                  NOVO_CODIGO('ITDETALORCAMESGOP', V_ITDETALORCAMESGOP);

                  INSERT INTO ITDETALORCAMESGOP
                              (NRITORCAMESGOP, NRITDETALORCAMESGOP, VRORCAMENTO, VRPERCENTUAL, QTORCAMENTO, CDPRODUTO)
                       VALUES (V_NRITORCAMESGOP, V_ITDETALORCAMESGOP, 0, 0, 0, C_DETALHAMESFIL.CDPRODUTO);
               END LOOP;

               FOR C_DETALHAMES IN (SELECT PR.CDPRODUTO
                                      FROM PERCITEMGOP P INNER JOIN PERCITEMDETALGOP PD
                                                                 ON P.NRPERCITEMGOP = PD.NRPERCITEMGOP
                                                         INNER JOIN PRODDESPPREV PR
                                                                 ON PD.CDDESPPREV = PR.CDDESPPREV
                                     WHERE P.NRSEQITEMORCA = P_ITEM
                                       AND PD.IDDESPPREV = V_DESPESA
                                       AND NVL(PD.CDDESPPREV,'T') = NVL(P_CDDESPPREV,'T')
                                       AND NOT EXISTS (SELECT 1
                                                         FROM PERCITFILGOP PF INNER JOIN PERCITDETALFILGOP PDF
                                                                                      ON PF.NRPERCITFILGOP = PDF.NRPERCITFILGOP
                                                                              INNER JOIN PERCDETPRODFILGOP PPF
                                                                                      ON PDF.NRPERCITDETALFILGOP = PPF.NRPERCITDETALFILGOP
                                                        WHERE PF.NRSEQITEMORCA = P_ITEM
                                                          AND PDF.IDDESPPREV = V_DESPESA
                                                          AND PF.CDFILIAL = P_CDFILIAL    
                                                          AND NVL(PDF.CDDESPPREV,'T') = NVL(P_CDDESPPREV,'T')
                                                          AND PPF.CDPRODUTO = PR.CDPRODUTO)
                                     MINUS
                                    SELECT IT.CDPRODUTO
                                      FROM ITEMORCAGOP I INNER JOIN ITORCAMESGOP IM ON I.NRITEMORCAOP = IM.NRITEMORCAOP
                                                         INNER JOIN ITDETALORCAMESGOP IT ON IM.NRITORCAMESGOP = IT.NRITORCAMESGOP
                                     WHERE IT.NRITORCAMESGOP = V_NRITORCAMESGOP
                                       AND I.IDTIPOORCAOP = V_ORCAMENTO) LOOP

                  NOVO_CODIGO('ITDETALORCAMESGOP', V_ITDETALORCAMESGOP);

                  INSERT INTO ITDETALORCAMESGOP
                              (NRITORCAMESGOP, NRITDETALORCAMESGOP, VRORCAMENTO, VRPERCENTUAL, QTORCAMENTO, CDPRODUTO)
                       VALUES (V_NRITORCAMESGOP, V_ITDETALORCAMESGOP, 0, 0, 0, C_DETALHAMES.CDPRODUTO);
               END LOOP;
            ELSIF P_ITEM IN (2) THEN
               FOR C_DETALHAMESFIL IN (SELECT PD.CDPRODUTO
                                         FROM PERCITFILGOP P INNER JOIN PERCITDETALFILGOP PD ON P.NRPERCITFILGOP = PD.NRPERCITFILGOP
                                        WHERE P.CDFILIAL = P_CDFILIAL
                                          AND P.NRSEQITEMORCA = P_ITEM
                                        MINUS
                                       SELECT IT.CDPRODUTO
                                         FROM ITEMORCAGOP I INNER JOIN ITORCAMESGOP IM ON I.NRITEMORCAOP = IM.NRITEMORCAOP
                                                            INNER JOIN ITDETALORCAMESGOP IT ON IM.NRITORCAMESGOP = IT.NRITORCAMESGOP
                                        WHERE IT.NRITORCAMESGOP = V_NRITORCAMESGOP
                                          AND I.IDTIPOORCAOP = V_ORCAMENTO) LOOP

                  NOVO_CODIGO('ITDETALORCAMESGOP', V_ITDETALORCAMESGOP);

                  INSERT INTO ITDETALORCAMESGOP
                              (NRITORCAMESGOP, NRITDETALORCAMESGOP, VRORCAMENTO, VRPERCENTUAL, QTORCAMENTO, CDPRODUTO)
                       VALUES (V_NRITORCAMESGOP, V_ITDETALORCAMESGOP, 0, 0, 0, C_DETALHAMESFIL.CDPRODUTO);
               END LOOP;

               FOR C_DETALHAMES IN (SELECT PD.CDPRODUTO
                                      FROM PERCITEMGOP P INNER JOIN PERCITEMDETALGOP PD ON P.NRPERCITEMGOP = PD.NRPERCITEMGOP
                                     WHERE P.NRSEQITEMORCA = P_ITEM
                                       AND NOT EXISTS (SELECT 1
                                                         FROM PERCITFILGOP PF INNER JOIN PERCITDETALFILGOP PDF
                                                                                      ON PF.NRPERCITFILGOP = PDF.NRPERCITFILGOP
                                                        WHERE PF.CDFILIAL = P_CDFILIAL
                                                          AND PF.NRPERCITFILGOP = P_ITEM
                                                          AND PDF.CDPRODUTO = PD.CDPRODUTO)
                                     MINUS
                                    SELECT IT.CDPRODUTO
                                      FROM ITEMORCAGOP I INNER JOIN ITORCAMESGOP IM ON I.NRITEMORCAOP = IM.NRITEMORCAOP
                                                         INNER JOIN ITDETALORCAMESGOP IT ON IM.NRITORCAMESGOP = IT.NRITORCAMESGOP
                                     WHERE IT.NRITORCAMESGOP = V_NRITORCAMESGOP
                                       AND I.IDTIPOORCAOP = V_ORCAMENTO) LOOP

                  NOVO_CODIGO('ITDETALORCAMESGOP', V_ITDETALORCAMESGOP);

                  INSERT INTO ITDETALORCAMESGOP
                              (NRITORCAMESGOP, NRITDETALORCAMESGOP, VRORCAMENTO, VRPERCENTUAL, QTORCAMENTO, CDPRODUTO)
                       VALUES (V_NRITORCAMESGOP, V_ITDETALORCAMESGOP, 0, 0, 0, C_DETALHAMES.CDPRODUTO);
               END LOOP;
            END IF;
         END IF;
      END LOOP;
   END;

   FUNCTION STRPERIODO (NRPERIODOORCA NUMBER, IDPERIODOORCA VARCHAR2) RETURN VARCHAR2 AS
     V_DESCNRPERIODOORCA VARCHAR2(40);
   BEGIN
      IF TO_CHAR(IDPERIODOORCA) = 'MENSAL' THEN
         V_DESCNRPERIODOORCA := CASE NRPERIODOORCA WHEN 1 THEN 'Janeiro'
                                                   WHEN 2 THEN 'Fevereiro'
                                                   WHEN 3 THEN 'Maro'
                                                   WHEN 4 THEN 'Abril'
                                                   WHEN 5 THEN 'Maio'
                                                   WHEN 6 THEN 'Junho'
                                                   WHEN 7 THEN 'Julho'
                                                   WHEN 8 THEN 'Agosto'
                                                   WHEN 9 THEN 'Setembro'
                                                   WHEN 10 THEN 'Outubro'
                                                   WHEN 11 THEN 'Novembro'
                                                   WHEN 12 THEN 'Dezembro'
                                                   ELSE ' '
                                END;
      ELSIF IDPERIODOORCA = 'BIMESTRAL' THEN
         V_DESCNRPERIODOORCA := CASE NRPERIODOORCA WHEN 1 THEN '1 Bimestre'
                                                   WHEN 2 THEN '2 Bimestre'
                                                   WHEN 3 THEN '3 Bimestre'
                                                   WHEN 4 THEN '4 Bimestre'
                                                   WHEN 5 THEN '5 Bimestre'
                                                   WHEN 6 THEN '6 Bimestre'
                                                   ELSE ' '
                                END;
      ELSIF IDPERIODOORCA = 'TRIMESTRAL' THEN
         V_DESCNRPERIODOORCA := CASE NRPERIODOORCA WHEN 1 THEN '1 Trimestre'
                                                   WHEN 2 THEN '2 Trimestre'
                                                   WHEN 3 THEN '3 Trimestre'
                                                   WHEN 4 THEN '4 Trimestre'
                                                   ELSE ' '
                                END;
      ELSIF IDPERIODOORCA = 'QUADRIMESTRAL' THEN
         V_DESCNRPERIODOORCA := CASE NRPERIODOORCA WHEN 1 THEN '1 Quadrimestre'
                                                   WHEN 2 THEN '2 Quadrimestre'
                                                   WHEN 3 THEN '3 Quadrimestre'
                                                   ELSE ' '
                                END;
      ELSIF IDPERIODOORCA = 'SEMESTRAL' THEN 
         V_DESCNRPERIODOORCA := CASE NRPERIODOORCA WHEN 1 THEN '1 Semestre'
                                                   WHEN 2 THEN '2 Semestre'
                                                   ELSE ' '
                                END;
      END IF;

      RETURN V_DESCNRPERIODOORCA;
   END;
BEGIN
   BEGIN
      SELECT IDUTPADRROP
        INTO V_IDUTPADRROP
        FROM PARAMGERAL G;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
         V_IDUTPADRROP := NULL;
      END;
   END;

   IF NVL(P_ORCADETA, 'N') = 'N' THEN
      IF (P_LISTAOPER = 'S') AND (P_LISTAFIL = 'S') THEN
         FOR C_ORCOP01 IN (SELECT NRANOORCAOP, CDVERSORCAOP, CDFILIAL, MAX(CDOPERADOR) AS CDOPERADOR
                             FROM (SELECT DISTINCT P_NRANOORCAOP AS NRANOORCAOP, P_CDVERSORCAOP AS CDVERSORCAOP, CDFILIAL, FO.CDOPERADOR
                                     FROM FILIALOPEROP FO, LISTA O, LISTA F
                                    WHERE O.NRLISTA       = P_CDOPERADOR
                                      AND F.NRLISTA       = P_CDFILIAL
                                      AND FO.CDFILIAL     = F.CDLISTA
                                      AND FO.CDOPERADOR   = O.CDLISTA
                                      AND FO.IDACESSOCALC = 'S')
                                    GROUP BY NRANOORCAOP, CDVERSORCAOP, CDFILIAL) LOOP

            BEGIN
               SELECT CDFILIAL, 'S',IDSTATUSORCA
                 INTO V_FILIAL, V_EXISTE,V_IDSTATUSORCA
                 FROM ORCAMENTOOP
                WHERE NRANOORCAOP  = C_ORCOP01.NRANOORCAOP
                  AND CDVERSORCAOP = C_ORCOP01.CDVERSORCAOP
                  AND CDFILIAL     = C_ORCOP01.CDFILIAL;
            EXCEPTION
                 WHEN NO_DATA_FOUND THEN BEGIN
                      V_FILIAL       := NULL;
                      V_EXISTE       := 'N';
                      V_IDSTATUSORCA := NULL;
                 END;
            END;

            IF V_EXISTE = 'S' THEN
               IF P_IDEXCLUI = 'S' AND V_IDSTATUSORCA <> 'A' THEN
                  UPDATE ORCAMENTOOP
                     SET IDSTATUSORCA = 'P',
                         DTORCAMENTO  = SYSDATE,
                         CDOPERADOR   = C_ORCOP01.CDOPERADOR
                   WHERE NRANOORCAOP  = C_ORCOP01.NRANOORCAOP
                     AND CDVERSORCAOP = C_ORCOP01.CDVERSORCAOP
                     AND CDFILIAL     = C_ORCOP01.CDFILIAL;

                  UPDATE ITEMORCAOP
                     SET IDITEMORCAMENTO = 'P'
                   WHERE NRANOORCAOP     = C_ORCOP01.NRANOORCAOP
                     AND CDVERSORCAOP    = C_ORCOP01.CDVERSORCAOP
                     AND CDFILIAL        = C_ORCOP01.CDFILIAL;
               END IF;
            ELSE
               INSERT INTO ORCAMENTOOP
                      (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, CDOPERADOR, DTORCAMENTO, IDSTATUSORCA)
               VALUES (C_ORCOP01.NRANOORCAOP, C_ORCOP01.CDVERSORCAOP, C_ORCOP01.CDFILIAL, C_ORCOP01.CDOPERADOR, SYSDATE, 'P');
            END IF;
         END LOOP;
      ELSE
         IF (P_LISTAOPER = 'S') AND (NVL(P_LISTAFIL,'S') = 'N') THEN
            FOR C_ORCOP02 IN (SELECT NRANOORCAOP, CDVERSORCAOP, CDFILIAL, MAX(CDOPERADOR) AS CDOPERADOR
                                FROM (SELECT DISTINCT P_NRANOORCAOP AS NRANOORCAOP, P_CDVERSORCAOP AS CDVERSORCAOP, CDFILIAL, FO.CDOPERADOR
                                        FROM FILIALOPEROP FO, LISTA O
                                       WHERE O.NRLISTA       = P_CDOPERADOR
                                         AND FO.CDOPERADOR   = O.CDLISTA
                                         AND FO.IDACESSOCALC = 'S')
                              GROUP BY NRANOORCAOP, CDVERSORCAOP, CDFILIAL) LOOP

               BEGIN
                  SELECT CDFILIAL, 'S',IDSTATUSORCA
                    INTO V_FILIAL,V_EXISTE,V_IDSTATUSORCA
                    FROM ORCAMENTOOP
                   WHERE NRANOORCAOP  = C_ORCOP02.NRANOORCAOP
                     AND CDVERSORCAOP = C_ORCOP02.CDVERSORCAOP
                     AND CDFILIAL     = C_ORCOP02.CDFILIAL;
               EXCEPTION
                  WHEN NO_DATA_FOUND THEN BEGIN
                     V_FILIAL := NULL;
                     V_EXISTE := 'N';
                     V_IDSTATUSORCA := NULL;
                  END;
               END;

               IF V_EXISTE = 'S' THEN
                  IF P_IDEXCLUI = 'S' AND V_IDSTATUSORCA <> 'A' THEN
                     UPDATE ORCAMENTOOP
                        SET IDSTATUSORCA = 'P',
                            DTORCAMENTO  = SYSDATE,
                            CDOPERADOR   = C_ORCOP02.CDOPERADOR
                      WHERE NRANOORCAOP  = C_ORCOP02.NRANOORCAOP
                        AND CDVERSORCAOP = C_ORCOP02.CDVERSORCAOP
                        AND CDFILIAL     = C_ORCOP02.CDFILIAL;

                     UPDATE ITEMORCAOP
                        SET IDITEMORCAMENTO = 'P'
                      WHERE NRANOORCAOP  = C_ORCOP02.NRANOORCAOP
                        AND CDVERSORCAOP = C_ORCOP02.CDVERSORCAOP
                        AND CDFILIAL     = C_ORCOP02.CDFILIAL;
                  END IF;
               ELSE
                  INSERT INTO ORCAMENTOOP
                         (NRANOORCAOP, CDVERSORCAOP, CDFILIAL,CDOPERADOR,DTORCAMENTO, IDSTATUSORCA)
                  VALUES (C_ORCOP02.NRANOORCAOP,C_ORCOP02.CDVERSORCAOP,C_ORCOP02.CDFILIAL, C_ORCOP02.CDOPERADOR, SYSDATE, 'P');
               END IF;
            END LOOP;
         ELSE
            IF (P_LISTAOPER = 'N') AND (NVL(P_LISTAFIL,'S') = 'S') THEN
               FOR C_ORCOP03 IN (SELECT NRANOORCAOP, CDVERSORCAOP, CDFILIAL, MAX(CDOPERADOR) AS CDOPERADOR
                                   FROM (SELECT DISTINCT P_NRANOORCAOP AS NRANOORCAOP, P_CDVERSORCAOP AS CDVERSORCAOP, CDFILIAL, FO.CDOPERADOR
                                           FROM FILIALOPEROP FO, LISTA F
                                          WHERE F.NRLISTA       = P_CDFILIAL
                                            AND FO.CDFILIAL     = F.CDLISTA
                                            AND FO.IDACESSOCALC = 'S'
                                            AND FO.CDOPERADOR   = P_CDOPERADOR)
                                 GROUP BY NRANOORCAOP, CDVERSORCAOP, CDFILIAL) LOOP

                  BEGIN
                     SELECT CDFILIAL, 'S',IDSTATUSORCA
                       INTO V_FILIAL, V_EXISTE,V_IDSTATUSORCA
                       FROM ORCAMENTOOP
                      WHERE NRANOORCAOP  = C_ORCOP03.NRANOORCAOP
                        AND CDVERSORCAOP = C_ORCOP03.CDVERSORCAOP
                        AND CDFILIAL     = C_ORCOP03.CDFILIAL;
                  EXCEPTION
                     WHEN NO_DATA_FOUND THEN BEGIN
                        V_FILIAL       := NULL;
                        V_EXISTE       := 'N';
                        V_IDSTATUSORCA := NULL;
                     END;
                  END;

                  IF V_EXISTE = 'S' THEN
                     IF P_IDEXCLUI = 'S' AND V_IDSTATUSORCA <> 'A' THEN
                        UPDATE ORCAMENTOOP
                           SET IDSTATUSORCA = 'P',
                               DTORCAMENTO  = SYSDATE,
                               CDOPERADOR   = C_ORCOP03.CDOPERADOR
                         WHERE NRANOORCAOP  = C_ORCOP03.NRANOORCAOP
                           AND CDVERSORCAOP = C_ORCOP03.CDVERSORCAOP
                           AND CDFILIAL     = C_ORCOP03.CDFILIAL;

                        UPDATE ITEMORCAOP
                           SET IDITEMORCAMENTO = 'P'
                         WHERE NRANOORCAOP  = C_ORCOP03.NRANOORCAOP
                           AND CDVERSORCAOP = C_ORCOP03.CDVERSORCAOP
                           AND CDFILIAL     = C_ORCOP03.CDFILIAL;
                     END IF;
                  ELSE
                     INSERT INTO ORCAMENTOOP
                            (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, CDOPERADOR, DTORCAMENTO, IDSTATUSORCA)
                     VALUES (C_ORCOP03.NRANOORCAOP, C_ORCOP03.CDVERSORCAOP, C_ORCOP03.CDFILIAL, C_ORCOP03.CDOPERADOR, SYSDATE, 'P');
                  END IF;
               END LOOP;
            ELSE

               FOR C_ORCOP04 IN (SELECT NRANOORCAOP, CDVERSORCAOP, CDFILIAL, MAX(CDOPERADOR) AS CDOPERADOR
                                   FROM (SELECT DISTINCT P_NRANOORCAOP AS NRANOORCAOP, P_CDVERSORCAOP AS CDVERSORCAOP, CDFILIAL, FO.CDOPERADOR
                                           FROM FILIALOPEROP FO
                                          WHERE FO.IDACESSOCALC = 'S'
                                            AND FO.CDOPERADOR   = P_CDOPERADOR)
                                 GROUP BY NRANOORCAOP, CDVERSORCAOP, CDFILIAL) LOOP

                  BEGIN
                     SELECT CDFILIAL, 'S',IDSTATUSORCA
                       INTO V_FILIAL, V_EXISTE,V_IDSTATUSORCA
                       FROM ORCAMENTOOP
                      WHERE NRANOORCAOP  = C_ORCOP04.NRANOORCAOP
                        AND CDVERSORCAOP = C_ORCOP04.CDVERSORCAOP
                        AND CDFILIAL     = C_ORCOP04.CDFILIAL;
                  EXCEPTION
                     WHEN NO_DATA_FOUND THEN BEGIN
                        V_FILIAL := NULL;
                        V_EXISTE := 'N';
                        V_IDSTATUSORCA := NULL;
                     END;
                  END;

                  IF V_EXISTE = 'S' THEN

                     IF P_IDEXCLUI = 'S' AND V_IDSTATUSORCA <> 'A'THEN
                        UPDATE ORCAMENTOOP
                           SET IDSTATUSORCA = 'P',
                               DTORCAMENTO  = SYSDATE,
                               CDOPERADOR   = C_ORCOP04.CDOPERADOR
                         WHERE NRANOORCAOP  = C_ORCOP04.NRANOORCAOP
                           AND CDVERSORCAOP = C_ORCOP04.CDVERSORCAOP
                           AND CDFILIAL     = C_ORCOP04.CDFILIAL;

                        UPDATE ITEMORCAOP
                           SET IDITEMORCAMENTO = 'P'
                         WHERE NRANOORCAOP  = C_ORCOP04.NRANOORCAOP
                           AND CDVERSORCAOP = C_ORCOP04.CDVERSORCAOP
                           AND CDFILIAL     = C_ORCOP04.CDFILIAL;
                     END IF;

                  ELSE
                     INSERT INTO ORCAMENTOOP
                            (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, CDOPERADOR, DTORCAMENTO, IDSTATUSORCA)
                     VALUES (C_ORCOP04.NRANOORCAOP, C_ORCOP04.CDVERSORCAOP, C_ORCOP04.CDFILIAL, C_ORCOP04.CDOPERADOR, SYSDATE, 'P');

                  END IF;
               END LOOP;
            END IF;
         END IF;
      END IF;

      FOR C_FILIAL IN (SELECT O.CDFILIAL,O.IDSTATUSORCA
                         FROM ORCAMENTOOP O
                        WHERE O.NRANOORCAOP  = P_NRANOORCAOP
                          AND O.CDVERSORCAOP = P_CDVERSORCAOP
                          AND O.IDSTATUSORCA <> 'A'
                          AND ((O.CDFILIAL IN ( SELECT CDLISTA
                                                  FROM LISTA
                                                 WHERE NRLISTA = P_CDFILIAL ) ) OR (P_CDFILIAL IS NULL))) LOOP

           IF C_FILIAL.IDSTATUSORCA <> 'A' THEN

               IF (( C_FILIAL.IDSTATUSORCA <> 'E') OR (C_FILIAL.IDSTATUSORCA = 'E' AND P_IDEXCLUI = 'S')) THEN

                     IF (V_IDUTPADRROP = 'S') THEN
                        V_FILIALPAR := 'P';
                     ELSE
                        BEGIN
                           SELECT IDUTPADRROPFIL, CDFILCENTPME
                             INTO V_IDUTPADRROPFIL, V_CDFILCENTPME
                             FROM FILIAL F LEFT JOIN PARFILIALROP P ON F.CDFILIAL = P.CDFILIAL
                            WHERE F.CDFILIAL = C_FILIAL.CDFILIAL;
                        EXCEPTION
                          WHEN NO_DATA_FOUND THEN BEGIN
                            V_IDUTPADRROPFIL := NULL;
                            V_CDFILCENTPME   := NULL;
                          END;
                        END;
                        IF (V_IDUTPADRROPFIL = 'S') THEN
                           V_FILIALPAR := 'P';
                        ELSE
                           IF V_CDFILCENTPME IS NULL THEN
                              V_FILIALPAR := C_FILIAL.CDFILIAL;
                           ELSE
                              BEGIN
                                 SELECT IDFILIALAGRROP
                                   INTO V_IDFILIALAGRROP
                                   FROM PARFILIALROP
                                  WHERE CDFILIAL = V_CDFILCENTPME;
                              EXCEPTION
                                WHEN NO_DATA_FOUND THEN
                                  V_IDFILIALAGRROP := 'N';
                              END;
                              IF V_IDFILIALAGRROP = 'S' THEN
                                 V_FILIALPAR := V_CDFILCENTPME;
                              ELSE
                                 V_FILIALPAR := C_FILIAL.CDFILIAL;
                              END IF;
                           END IF;
                        END IF;
                     END IF;
                     BEGIN
                        SELECT IDFATUROP, IDSERVFATROP
                          INTO V_IDFATUROP, V_IDSERVFATROP
                          FROM PARFILIALROP P
                         WHERE P.CDFILIAL = V_FILIALPAR;
                     EXCEPTION
                       WHEN NO_DATA_FOUND THEN BEGIN
                         V_IDFATUROP    := NULL;
                         V_IDSERVFATROP := NULL;
                       END;
                     END;
                     SELECT LPAD((TO_NUMBER(NVL(MAX(NRSEQITEMORCA),'0')) + 1), 10, 0)
                       INTO V_NRITEMORCA
                       FROM ITEMORCAOP
                      WHERE CDFILIAL     = C_FILIAL.CDFILIAL
                        AND NRANOORCAOP  = P_NRANOORCAOP
                        AND CDVERSORCAOP = P_CDVERSORCAOP;

                     FOR C_SERVICO IN(  SELECT SC.CDSERVFILI
                                            FROM SERVICOROP SC, SERVICO S
                                             WHERE SC.CDFILIAL = TIPOPARAM_DRO(C_FILIAL.CDFILIAL)
                                             AND (IDTPSERVICOROP   = 'E')
                                             AND (V_IDSERVFATROP <> 'S')
                                             AND SC.CDSERVFILI = S.CDSERVFILI
                                             AND S.CDFILIAL = C_FILIAL.CDFILIAL
                                           UNION ALL
                                           SELECT CDSERVFILI
                                           FROM SERVICO S
                                           WHERE (S.CDFILIAL = C_FILIAL.CDFILIAL)
                                             AND (V_IDSERVFATROP   = 'S')
                                             AND (QTCOMESTISEG IS NOT  NULL OR
                                                  QTCOMESTITER IS NOT  NULL OR
                                                  QTCOMESTIQUA IS NOT  NULL OR
                                                  QTCOMESTIQUI IS NOT  NULL OR
                                                  QTCOMESTISEX IS NOT  NULL OR
                                                  QTCOMESTISAB IS NOT  NULL OR
                                                  QTCOMESTIDOM IS NOT  NULL OR
                                                  QTCOMESTIFER IS NOT  NULL OR
                                                  'S' = 'S'))  LOOP

                     INSERT INTO ITEMORCAOP
                      (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, NRSEQITEMORCA, IDENTRSAIDOP, CDTIPOOPER, CDIMPOSTO, CDDESPADMFAT, CDDESPPREV, DTORCAMENTOITEM, IDTIPOORCAOP, IDITEMORCAMENTO, CDSERVFILI)
                     (SELECT  P_NRANOORCAOP, P_CDVERSORCAOP, C_FILIAL.CDFILIAL, V_NRITEMORCA, '','', '', '','',SYSDATE, 'F','P', C_SERVICO.CDSERVFILI
                        FROM DUAL
                      MINUS
                      SELECT P_NRANOORCAOP, P_CDVERSORCAOP, C_FILIAL.CDFILIAL, V_NRITEMORCA, '','', '', '', '',SYSDATE, 'F','P', C_SERVICO.CDSERVFILI
                        FROM ITEMORCAOP
                       WHERE NRANOORCAOP  = P_NRANOORCAOP
                         AND CDVERSORCAOP = P_CDVERSORCAOP
                         AND CDFILIAL = C_FILIAL.CDFILIAL
                         AND IDTIPOORCAOP = 'F'
                         AND CDSERVFILI = C_SERVICO.CDSERVFILI);

                     V_NRITEMORCA := LPAD((TO_NUMBER(NVL(V_NRITEMORCA,'0')) + 1), 10, 0);


                     INSERT INTO ITEMORCAOP
                      (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, NRSEQITEMORCA, IDENTRSAIDOP, CDTIPOOPER, CDIMPOSTO, CDDESPADMFAT, CDDESPPREV, DTORCAMENTOITEM, IDTIPOORCAOP, IDITEMORCAMENTO, CDSERVFILI)
                     (SELECT P_NRANOORCAOP, P_CDVERSORCAOP, C_FILIAL.CDFILIAL, V_NRITEMORCA, '','', '', '', '',SYSDATE, 'C','P', C_SERVICO.CDSERVFILI
                        FROM DUAL
                       MINUS
                      SELECT P_NRANOORCAOP, P_CDVERSORCAOP, C_FILIAL.CDFILIAL, V_NRITEMORCA, '','', '', '', '',SYSDATE, 'C','P', C_SERVICO.CDSERVFILI
                        FROM ITEMORCAOP
                       WHERE NRANOORCAOP = P_NRANOORCAOP
                         AND CDVERSORCAOP = P_CDVERSORCAOP
                         AND CDFILIAL = C_FILIAL.CDFILIAL
                         AND IDTIPOORCAOP = 'C'
                         AND CDSERVFILI = C_SERVICO.CDSERVFILI );

                         V_NRITEMORCA := LPAD((TO_NUMBER(NVL(V_NRITEMORCA,'0')) + 1), 10, 0);

                    END LOOP;

                    FOR C_IMPOSTO IN (SELECT DISTINCT DECODE(CDFILIAL, 'T',  C_FILIAL.CDFILIAL , CDFILIAL) AS CDFILIAL , CDIMPOSTO
                                        FROM IMPOSTOORCAFIL
                                       WHERE ((CDFILIAL = C_FILIAL.CDFILIAL) OR (CDFILIAL = 'T'))
                                         AND VRPERCSOBFAT  <> 0
                                      MINUS
                                      SELECT C_FILIAL.CDFILIAL, CDIMPOSTO
                                        FROM ITEMORCAOP
                                       WHERE NRANOORCAOP  = P_NRANOORCAOP
                                         AND CDVERSORCAOP = P_CDVERSORCAOP
                                         AND CDFILIAL = C_FILIAL.CDFILIAL
                                         AND IDTIPOORCAOP = 'I') LOOP
                             V_NRITEMORCA := LPAD((TO_NUMBER(NVL(V_NRITEMORCA,'0')) + 1), 10, 0);
                             INSERT INTO ITEMORCAOP
                              (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, NRSEQITEMORCA, IDENTRSAIDOP, CDTIPOOPER, CDIMPOSTO ,CDDESPADMFAT, CDDESPPREV, DTORCAMENTOITEM, IDTIPOORCAOP, IDITEMORCAMENTO)
                             VALUES
                              (P_NRANOORCAOP, P_CDVERSORCAOP, C_IMPOSTO.CDFILIAL, V_NRITEMORCA, '','', C_IMPOSTO.CDIMPOSTO,'','',SYSDATE, 'I','P');
                    END LOOP;

                    FOR C_IMPOSTOREC IN (SELECT DISTINCT DECODE(CDFILIAL, 'T',  C_FILIAL.CDFILIAL , CDFILIAL) AS CDFILIAL, CDIMPOSTO
                                           FROM IMPOSTOORCAFIL
                                          WHERE ((CDFILIAL = C_FILIAL.CDFILIAL) OR (CDFILIAL = 'T'))
                                            AND VRPERCSOBCONS  <> 0
                                         MINUS
                                         SELECT C_FILIAL.CDFILIAL, CDIMPOSTO
                                           FROM ITEMORCAOP
                                          WHERE NRANOORCAOP  = P_NRANOORCAOP
                                            AND CDVERSORCAOP = P_CDVERSORCAOP
                                            AND CDFILIAL = C_FILIAL.CDFILIAL
                                            AND IDTIPOORCAOP = 'R') LOOP
                             V_NRITEMORCA := LPAD((TO_NUMBER(NVL(V_NRITEMORCA,'0')) + 1), 10, 0);
                             INSERT INTO ITEMORCAOP
                              (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, NRSEQITEMORCA, IDENTRSAIDOP, CDTIPOOPER, CDIMPOSTO ,CDDESPADMFAT, CDDESPPREV, DTORCAMENTOITEM, IDTIPOORCAOP, IDITEMORCAMENTO)
                             VALUES
                              (P_NRANOORCAOP, P_CDVERSORCAOP, C_IMPOSTOREC.CDFILIAL, V_NRITEMORCA, '','',  C_IMPOSTOREC.CDIMPOSTO,'','',SYSDATE, 'R','P');
                    END LOOP;

                    FOR C_IMPOSLUCR IN (SELECT DISTINCT DECODE(CDFILIAL, 'T',  C_FILIAL.CDFILIAL , CDFILIAL) AS CDFILIAL
                                           FROM IMPOSTOORCAFIL
                                          WHERE CDFILIAL = C_FILIAL.CDFILIAL OR CDFILIAL = 'T'
                                         MINUS
                                         SELECT C_FILIAL.CDFILIAL
                                           FROM ITEMORCAOP
                                          WHERE NRANOORCAOP  = P_NRANOORCAOP
                                            AND CDVERSORCAOP = P_CDVERSORCAOP
                                            AND CDFILIAL = C_FILIAL.CDFILIAL
                                            AND IDTIPOORCAOP = 'S') LOOP
                             V_NRITEMORCA := LPAD((TO_NUMBER(NVL(V_NRITEMORCA,'0')) + 1), 10, 0);
                             INSERT INTO ITEMORCAOP
                              (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, NRSEQITEMORCA, IDENTRSAIDOP, CDTIPOOPER, CDIMPOSTO ,CDDESPADMFAT, CDDESPPREV, DTORCAMENTOITEM, IDTIPOORCAOP, IDITEMORCAMENTO)
                             VALUES
                              (P_NRANOORCAOP, P_CDVERSORCAOP, C_IMPOSLUCR.CDFILIAL, V_NRITEMORCA, '','', '','','',SYSDATE, 'S','P');
                    END LOOP;

                    FOR C_INVEST IN (SELECT DISTINCT DECODE(CDFILIAL, 'T',  C_FILIAL.CDFILIAL , CDFILIAL) AS CDFILIAL
                                           FROM ITEMORCAOP
                                          WHERE CDFILIAL = C_FILIAL.CDFILIAL
                                         MINUS
                                         SELECT C_FILIAL.CDFILIAL
                                           FROM ITEMORCAOP
                                          WHERE NRANOORCAOP  = P_NRANOORCAOP
                                            AND CDVERSORCAOP = P_CDVERSORCAOP
                                            AND CDFILIAL = C_FILIAL.CDFILIAL
                                            AND IDTIPOORCAOP = 'N') LOOP
                             V_NRITEMORCA := LPAD((TO_NUMBER(NVL(V_NRITEMORCA,'0')) + 1), 10, 0);
                             INSERT INTO ITEMORCAOP
                              (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, NRSEQITEMORCA, IDENTRSAIDOP, CDTIPOOPER, CDIMPOSTO ,CDDESPADMFAT, CDDESPPREV, DTORCAMENTOITEM, IDTIPOORCAOP, IDITEMORCAMENTO)
                             VALUES
                              (P_NRANOORCAOP, P_CDVERSORCAOP, C_INVEST.CDFILIAL, V_NRITEMORCA, '','', '','','',SYSDATE, 'N','P');

                    END LOOP;

                    FOR C_DEPREC IN (SELECT DISTINCT DECODE(CDFILIAL, 'T',  C_FILIAL.CDFILIAL , CDFILIAL) AS CDFILIAL
                                           FROM ITEMORCAOP
                                          WHERE CDFILIAL = C_FILIAL.CDFILIAL
                                         MINUS
                                         SELECT C_FILIAL.CDFILIAL
                                           FROM ITEMORCAOP
                                          WHERE NRANOORCAOP  = P_NRANOORCAOP
                                            AND CDVERSORCAOP = P_CDVERSORCAOP
                                            AND CDFILIAL = C_FILIAL.CDFILIAL
                                            AND IDTIPOORCAOP = 'P') LOOP
                             V_NRITEMORCA := LPAD((TO_NUMBER(NVL(V_NRITEMORCA,'0')) + 1), 10, 0);
                             INSERT INTO ITEMORCAOP
                              (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, NRSEQITEMORCA, IDENTRSAIDOP, CDTIPOOPER, CDIMPOSTO ,CDDESPADMFAT, CDDESPPREV, DTORCAMENTOITEM, IDTIPOORCAOP, IDITEMORCAMENTO)
                             VALUES
                              (P_NRANOORCAOP, P_CDVERSORCAOP, C_DEPREC.CDFILIAL, V_NRITEMORCA, '','', '','','',SYSDATE, 'P','P');
                    END LOOP;

                    /*Mão de obra em duas etapas. Primeiro os orçamentos POR FILIAL e depois os Gerais
                     1 - Orçamento de MO por filial*/
                    FOR C_MAOOBRA_FILIAL IN (
                                              SELECT C_FILIAL.CDFILIAL, CDDESPADMFAT
                                                FROM DESPADMFATUR
                                               MINUS
                                              SELECT C_FILIAL.CDFILIAL, CDDESPPREV
                                                FROM ITEMORCAOP
                                               WHERE NRANOORCAOP  = P_NRANOORCAOP
                                                 AND CDVERSORCAOP = P_CDVERSORCAOP
                                                 AND CDFILIAL     = C_FILIAL.CDFILIAL
                                                 AND IDTIPOORCAOP = 'O'
                                            ) LOOP
                       BEGIN
                         SELECT COUNT(CDORCAMENTOMO) AS ORCAFILI
                           INTO V_EXIST_ORCFIL
                           FROM ORCAMENTOMO
                          WHERE CDFILIAL = C_MAOOBRA_FILIAL.CDFILIAL;
                       EXCEPTION
                          WHEN NO_DATA_FOUND THEN
                             V_EXIST_ORCFIL := 0;
                       END;

                       /*Verifica se existe MO por filial*/
                       IF V_EXIST_ORCFIL > 0 THEN
                          BEGIN
                             SELECT COUNT(CDORCAMENTOMO)
                               INTO V_EXIST_MOFIL
                               FROM DESPORCAMO
                              WHERE CDFILIAL = C_MAOOBRA_FILIAL.CDFILIAL
                                AND CDDESPADMFAT = C_MAOOBRA_FILIAL.CDDESPADMFAT;
                          EXCEPTION
                             WHEN NO_DATA_FOUND THEN
                                V_EXIST_MOFIL := 0;
                          END;

                          IF V_EXIST_MOFIL > 0 THEN

                             FOR C_ITEMMO IN(SELECT DE.CDDESPADMFAT,DM.CDORCAMENTOMO
                                               FROM DESPORCAMO DM, DESPADMFATUR DE
                                              WHERE DM.CDDESPADMFAT = DE.CDDESPADMFAT
                                                AND DM.CDDESPADMFAT = C_MAOOBRA_FILIAL.CDDESPADMFAT
                                                AND CDFILIAL = C_MAOOBRA_FILIAL.CDFILIAL
                                             )LOOP

                                   V_NRITEMORCA := LPAD((TO_NUMBER(NVL(V_NRITEMORCA,'0')) + 1), 10, 0);
                                   INSERT INTO ITEMORCAOP
                                    (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, NRSEQITEMORCA, IDENTRSAIDOP, CDTIPOOPER, CDIMPOSTO, CDDESPADMFAT,CDORCAMENTOMO, CDDESPPREV, DTORCAMENTOITEM, IDTIPOORCAOP, IDITEMORCAMENTO)
                                   VALUES
                                    (P_NRANOORCAOP, P_CDVERSORCAOP, C_FILIAL.CDFILIAL, V_NRITEMORCA, '','', '',C_MAOOBRA_FILIAL.CDDESPADMFAT,C_ITEMMO.CDORCAMENTOMO,'',SYSDATE, 'O','P');
                             END LOOP;
                          ELSE
                             BEGIN
                                SELECT COUNT(CDORCAMENTOMO)
                                  INTO V_EXIST_MOGER
                                  FROM DESPORCAMO
                                 WHERE CDFILIAL = 'T'
                                   AND CDDESPADMFAT = C_MAOOBRA_FILIAL.CDDESPADMFAT;
                              EXCEPTION
                                 WHEN NO_DATA_FOUND THEN
                                    V_EXIST_MOGER := 0;
                              END;
                          END IF;
                       END IF;

                    END LOOP;

                    /*2 - Orçamento de MO Geral*/
                    FOR C_GERALMO IN( SELECT C_FILIAL.CDFILIAL,DE.CDDESPADMFAT,DM.CDORCAMENTOMO
                                       FROM DESPORCAMO DM, DESPADMFATUR DE
                                      WHERE DM.CDDESPADMFAT = DE.CDDESPADMFAT
                                        AND CDFILIAL = 'T'
                                     MINUS
                                     SELECT C_FILIAL.CDFILIAL, CDDESPADMFAT ,CDORCAMENTOMO
                                       FROM ITEMORCAOP
                                      WHERE NRANOORCAOP  = P_NRANOORCAOP
                                        AND CDVERSORCAOP = P_CDVERSORCAOP
                                        AND CDFILIAL     = C_FILIAL.CDFILIAL
                                        AND IDTIPOORCAOP = 'O'
                                     )LOOP

                                    SELECT COUNT(CDORCAMENTOMO)
                                      INTO V_EXIST_MOFIL
                                      FROM DESPORCAMO
                                     WHERE CDFILIAL     = C_FILIAL.CDFILIAL
                                       AND CDDESPADMFAT = C_GERALMO.CDDESPADMFAT;

                                   V_NRITEMORCA := LPAD((TO_NUMBER(NVL(V_NRITEMORCA,'0')) + 1), 10, 0);
                                   INSERT INTO ITEMORCAOP
                                    (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, NRSEQITEMORCA, IDENTRSAIDOP, CDTIPOOPER, CDIMPOSTO, CDDESPADMFAT,CDORCAMENTOMO, CDDESPPREV, DTORCAMENTOITEM, IDTIPOORCAOP, IDITEMORCAMENTO)
                                   VALUES
                                    (P_NRANOORCAOP, P_CDVERSORCAOP, C_FILIAL.CDFILIAL, V_NRITEMORCA, '','', '', C_GERALMO.CDDESPADMFAT, C_GERALMO.CDORCAMENTOMO,'',SYSDATE, 'O','P');

                    END LOOP;

                    FOR C_TAXAADM IN(SELECT DISTINCT DECODE(CDFILIAL, 'T',  C_FILIAL.CDFILIAL , CDFILIAL) AS CDFILIAL
                                           FROM ITEMORCAOP
                                          WHERE CDFILIAL = C_FILIAL.CDFILIAL
                                          MINUS
                                         SELECT C_FILIAL.CDFILIAL
                                           FROM ITEMORCAOP
                                          WHERE NRANOORCAOP  = P_NRANOORCAOP
                                            AND CDVERSORCAOP = P_CDVERSORCAOP
                                            AND CDFILIAL = C_FILIAL.CDFILIAL
                                            AND IDTIPOORCAOP = 'T') LOOP
                             V_NRITEMORCA := LPAD((TO_NUMBER(NVL(V_NRITEMORCA,'0')) + 1), 10, 0);
                             INSERT INTO ITEMORCAOP
                              (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, NRSEQITEMORCA, IDENTRSAIDOP, CDTIPOOPER, CDIMPOSTO ,CDDESPADMFAT, CDDESPPREV, DTORCAMENTOITEM, IDTIPOORCAOP, IDITEMORCAMENTO)
                             VALUES
                              (P_NRANOORCAOP, P_CDVERSORCAOP, C_TAXAADM.CDFILIAL, V_NRITEMORCA, '','', '','','',SYSDATE, 'T','P');

                    END LOOP;

                    FOR C_DIVERSAS IN (SELECT C_FILIAL.CDFILIAL, CDDESPPREV
                                          FROM DESPPREV
                                         WHERE IDDESPESAMATRIZ <> 'S'
                                           AND IDDESPLANENTR <> 'S'
                                           AND IDATIVO = 'S'
                                         MINUS
                                        SELECT C_FILIAL.CDFILIAL,CDDESPPREV
                                          FROM ITEMORCAOP
                                         WHERE NRANOORCAOP  = P_NRANOORCAOP
                                           AND CDVERSORCAOP = P_CDVERSORCAOP
                                           AND CDFILIAL     = C_FILIAL.CDFILIAL
                                           AND IDTIPOORCAOP IN ('D')) LOOP
                       V_NRITEMORCA := LPAD((TO_NUMBER(NVL(V_NRITEMORCA,'0')) + 1), 10, 0);
                       INSERT INTO ITEMORCAOP
                        (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, NRSEQITEMORCA, IDENTRSAIDOP, CDTIPOOPER, CDIMPOSTO, CDDESPADMFAT, CDDESPPREV, DTORCAMENTOITEM, IDTIPOORCAOP, IDITEMORCAMENTO)
                       VALUES
                        (P_NRANOORCAOP, P_CDVERSORCAOP, C_FILIAL.CDFILIAL, V_NRITEMORCA, '','', '', '', C_DIVERSAS.CDDESPPREV, SYSDATE, 'D','P');
                     END LOOP;

                     FOR C_MATRIZ IN (SELECT C_FILIAL.CDFILIAL, CDDESPPREV
                                        FROM DESPPREV
                                       WHERE IDDESPESAMATRIZ =  'S'
                                         AND IDDESPLANENTR   <> 'S'
                                         AND IDATIVO         =  'S'
                                       MINUS
                                      SELECT C_FILIAL.CDFILIAL, CDDESPPREV
                                        FROM ITEMORCAOP
                                       WHERE NRANOORCAOP  = P_NRANOORCAOP
                                         AND CDVERSORCAOP = P_CDVERSORCAOP
                                         AND CDFILIAL     = C_FILIAL.CDFILIAL
                                         AND IDTIPOORCAOP IN ('M')) LOOP

                       V_NRITEMORCA := LPAD((TO_NUMBER(NVL(V_NRITEMORCA,'0')) + 1), 10, 0);
                       INSERT INTO ITEMORCAOP
                        (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, NRSEQITEMORCA, IDENTRSAIDOP, CDTIPOOPER, CDIMPOSTO, CDDESPADMFAT, CDDESPPREV, DTORCAMENTOITEM, IDTIPOORCAOP, IDITEMORCAMENTO)
                       VALUES
                        (P_NRANOORCAOP, P_CDVERSORCAOP, C_FILIAL.CDFILIAL, V_NRITEMORCA, '','', '', '', C_MATRIZ.CDDESPPREV,SYSDATE, 'M','P');
                     END LOOP;

                     SELECT IDORCDESPENTROP
                       INTO V_ORCDESPENTROP
                       FROM PARAMORCAOPERAC;

                     IF V_ORCDESPENTROP = 'O' THEN

                        FOR C_ENTRADAS_0 IN (SELECT C_FILIAL.CDFILIAL, CDTIPOOPER
                                               FROM TIPOOPERACROP
                                              WHERE IDTPOPERACROP = 'DE'
                                                AND IDENTRSAIDOP  = 'E'
                                                AND CDFILIAL = V_FILIALPAR
                                           MINUS
                                             SELECT C_FILIAL.CDFILIAL, CDTIPOOPER
                                               FROM ITEMORCAOP
                                              WHERE NRANOORCAOP  = P_NRANOORCAOP
                                                AND CDVERSORCAOP = P_CDVERSORCAOP
                                                AND CDFILIAL = C_FILIAL.CDFILIAL
                                                AND IDTIPOORCAOP = 'E'
                                                AND IDENTRSAIDOP = 'E' )LOOP

                          V_NRITEMORCA := LPAD((TO_NUMBER(NVL(V_NRITEMORCA,'0')) + 1), 10, 0);
                          INSERT INTO ITEMORCAOP
                           (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, NRSEQITEMORCA, IDENTRSAIDOP, CDTIPOOPER, CDIMPOSTO, CDDESPADMFAT, CDDESPPREV, DTORCAMENTOITEM, IDTIPOORCAOP, IDITEMORCAMENTO)
                          VALUES
                           (P_NRANOORCAOP, P_CDVERSORCAOP, C_FILIAL.CDFILIAL, V_NRITEMORCA, 'E',C_ENTRADAS_0.CDTIPOOPER, '', '','',SYSDATE, 'E','P');
                        END LOOP;
                     ELSE
                        BEGIN
                           SELECT G.IDUTPADRROP, R.IDUTPADRROPFIL
                             INTO V_IDUTPADRROP, V_IDUTPADRROPFIL
                             FROM PARAMGERAL G, PARFILIALROP R
                            WHERE R.CDFILIAL = C_FILIAL.CDFILIAL;
                        EXCEPTION
                           WHEN NO_DATA_FOUND THEN BEGIN
                              V_IDUTPADRROP := 'S';
                              V_IDUTPADRROPFIL := 'S';
                           END;
                        END;

                        FOR C_ENTRADAS_D2 IN(

                            SELECT C_FILIAL.CDFILIAL,DP.CDDESPPREV
                              FROM DESPPREV DP
                             WHERE (DP.IDDESPESAMATRIZ <> 'S')
                               AND (DP.IDDESPLANENTR = 'S')
                               AND (DP.IDATIVO = 'S')
                             MINUS
                            SELECT C_FILIAL.CDFILIAL, CDDESPPREV
                              FROM ITEMORCAOP
                             WHERE NRANOORCAOP  = P_NRANOORCAOP
                               AND CDVERSORCAOP = P_CDVERSORCAOP
                               AND CDFILIAL     = C_FILIAL.CDFILIAL
                               AND IDTIPOORCAOP IN ('E')

                               ) LOOP
                             V_NRITEMORCA := LPAD((TO_NUMBER(NVL(V_NRITEMORCA,'0')) + 1), 10, 0);
                             INSERT INTO ITEMORCAOP
                             (NRANOORCAOP, CDVERSORCAOP, CDFILIAL, NRSEQITEMORCA, IDENTRSAIDOP, CDTIPOOPER, CDIMPOSTO, CDDESPADMFAT, CDDESPPREV, DTORCAMENTOITEM, IDTIPOORCAOP, IDITEMORCAMENTO)
                             VALUES
                             (P_NRANOORCAOP, P_CDVERSORCAOP, C_FILIAL.CDFILIAL, V_NRITEMORCA, '','', '', '', C_ENTRADAS_D2.CDDESPPREV, SYSDATE, 'E','P');
                  END LOOP;
               END IF;
            END IF;
         END IF;
      END LOOP;
   ELSE
      -- MOBILE
      FOR C_ORCAMENTO IN (SELECT F.CDFILIAL
                            FROM FILIAL F
                           WHERE EXISTS (SELECT 1
                                           FROM FILIALOPEROP FO INNER JOIN LISTA LF ON FO.CDOPERADOR = LF.CDLISTA
                                          WHERE LF.NRLISTA = P_CDOPERADOR
                                            AND FO.IDACESSOCALC = 'S'
                                            AND FO.CDFILIAL = F.CDFILIAL)
                             AND EXISTS (SELECT 1
                                           FROM FILIALOPEROP FO INNER JOIN LISTA LF ON FO.CDFILIAL = LF.CDLISTA
                                          WHERE LF.NRLISTA = P_CDFILIAL
                                            AND FO.IDACESSOCALC = 'S'
                                            AND FO.CDFILIAL = F.CDFILIAL)) LOOP

         C_ORCAMENTO_CDFILIAL := C_ORCAMENTO.CDFILIAL; 
         
         /*V_IDUTILPERCFILGOP: DETERMINA SE A UNIDADE VAI USAR O 
           PERCENTUAL GERAL OU POR FILIAL*/
         SELECT IDUTILPERCFILGOP
           INTO V_IDUTILPERCFILGOP
           FROM FILIAL
          WHERE CDFILIAL = C_ORCAMENTO_CDFILIAL; 

         SELECT NVL(MAX(IDPERIODOORCA), 'ANUAL')
           INTO V_IDPERIODOORCA
           FROM PARAMORCAOPERAC;

         DELETE FROM LOGPREPARAORCAGOP
               WHERE NRANOORCAGOP = P_NRANOORCAOP
                 AND IDPERIODOORCA = V_IDPERIODOORCA
                 AND NRPERIODOORCA = P_CDVERSORCAOP
                 AND CDFILIAL = C_ORCAMENTO.CDFILIAL;

         V_IDEXCLUI := P_IDEXCLUI;

         SELECT MAX(OG.IDPERIODOORCA)
           INTO V_OUTRPERIODO
           FROM ORCAMENTOGOP OG
          WHERE OG.CDFILIAL = C_ORCAMENTO.CDFILIAL
            AND OG.NRANOORCAGOP = P_NRANOORCAOP
            AND OG.IDPERIODOORCA <> V_IDPERIODOORCA;

         SELECT MIN(STATUS)
           INTO V_STATUSCONC
           FROM (SELECT NVL(MAX(NVL(CO.IDSTATUS,'PENDENTE')), 'PENDENTE') STATUS
                   FROM CALENPREORCAGOP CA LEFT JOIN CONCPREORCAGOP CO ON CO.NRCALENPREORCAGOP = CA.NRCALENPREORCAGOP
                                                                      AND CO.CDFILIAL = C_ORCAMENTO.CDFILIAL
                  WHERE CA.NRANOORCAGOP = P_NRANOORCAOP
                    AND CA.IDPERIODOORCA = V_IDPERIODOORCA
                    AND CA.NRPERIODOORCA = P_CDVERSORCAOP
                  UNION
                 SELECT NVL(MAX(NVL(CO.IDSTATUS,'PENDENTE')), 'PENDENTE') STATUS
                   FROM ORCAMENTOGOP OG INNER JOIN CONCPREORCAGOP CO ON OG.NRORCAMENTOGOP = CO.NRORCAMENTOGOP
                                                                    AND OG.CDFILIAL = CO.CDFILIAL
                  WHERE OG.CDFILIAL = C_ORCAMENTO.CDFILIAL
                    AND OG.NRANOORCAGOP = P_NRANOORCAOP
                    AND OG.IDPERIODOORCA = V_IDPERIODOORCA
                    AND OG.NRPERIODOORCA = P_CDVERSORCAOP
                    AND NOT EXISTS (SELECT 1
                                      FROM CALENPREORCAGOP CA
                                     WHERE CA.NRANOORCAGOP = OG.NRANOORCAGOP
                                       AND CA.IDPERIODOORCA = OG.IDPERIODOORCA
                                       AND CA.NRPERIODOORCA = OG.NRPERIODOORCA)) CONCLUSAO;

         BEGIN
            SELECT O.NRORCAMENTOGOP, NVL(A.IDAPROVACAOGOP, 'PENDENTE')
              INTO V_NRORCAMENTOGOP, V_IDAPROVACAOGOP
              FROM ORCAMENTOGOP O LEFT JOIN APROVAORCAGOP A ON A.NRORCAMENTOGOP = O.NRORCAMENTOGOP
             WHERE O.NRANOORCAGOP  = P_NRANOORCAOP
               AND O.NRPERIODOORCA = P_CDVERSORCAOP
               AND O.CDFILIAL      = C_ORCAMENTO.CDFILIAL;
         EXCEPTION
           WHEN NO_DATA_FOUND THEN BEGIN
              V_NRORCAMENTOGOP := NULL;
              V_IDAPROVACAOGOP := 'PENDENTE';
           END;
         END;

         IF (V_IDEXCLUI = 'S') AND (V_IDAPROVACAOGOP <> 'APROVADO') AND (P_ITEMORCA = 'SSSSSSSSSSSS') AND (V_STATUSCONC <> 'CONCLUIDO') THEN
            DELETE FROM ORCAMENTOGOP
                  WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP;

            V_NRORCAMENTOGOP := NULL;
         END IF;

         IF (V_IDAPROVACAOGOP <> 'APROVADO') AND (V_STATUSCONC <> 'CONCLUIDO') THEN
            IF (V_IDUTPADRROP = 'S') THEN
               V_FILIALPAR := 'P';
            ELSE
               BEGIN
                  SELECT IDUTPADRROPFIL, CDFILCENTPME
                    INTO V_IDUTPADRROPFIL, V_CDFILCENTPME
                    FROM FILIAL F LEFT JOIN PARFILIALROP P ON F.CDFILIAL = P.CDFILIAL
                   WHERE F.CDFILIAL = C_ORCAMENTO.CDFILIAL;
               EXCEPTION
                 WHEN NO_DATA_FOUND THEN BEGIN
                   V_IDUTPADRROPFIL := NULL;
                   V_CDFILCENTPME   := NULL;
                 END;
               END;
               IF (V_IDUTPADRROPFIL = 'S') THEN
                  V_FILIALPAR := 'P';
               ELSE
                  IF V_CDFILCENTPME IS NULL THEN
                     V_FILIALPAR := C_ORCAMENTO.CDFILIAL;
                  ELSE
                     BEGIN
                        SELECT IDFILIALAGRROP
                          INTO V_IDFILIALAGRROP
                          FROM PARFILIALROP
                         WHERE CDFILIAL = V_CDFILCENTPME;
                     EXCEPTION
                       WHEN NO_DATA_FOUND THEN
                         V_IDFILIALAGRROP := 'N';
                     END;
                     IF V_IDFILIALAGRROP = 'S' THEN
                        V_FILIALPAR := V_CDFILCENTPME;
                     ELSE
                        V_FILIALPAR := C_ORCAMENTO.CDFILIAL;
                     END IF;
                  END IF;
               END IF;
            END IF;
            BEGIN
               SELECT IDFATUROP, IDSERVFATROP
                 INTO V_IDFATUROP, V_IDSERVFATROP
                 FROM PARFILIALROP P
                WHERE P.CDFILIAL = V_FILIALPAR;
            EXCEPTION
              WHEN NO_DATA_FOUND THEN BEGIN
                V_IDFATUROP    := NULL;
                V_IDSERVFATROP := NULL;
              END;
            END;
                            
            IF V_NRORCAMENTOGOP IS NULL THEN
               V_EXISTEORCA := 'N';
               NOVO_CODIGO('ORCAMENTOGOP', V_NRORCAMENTOGOP);

               INSERT INTO ORCAMENTOGOP
                          (NRORCAMENTOGOP, NRANOORCAGOP, IDPERIODOORCA, NRPERIODOORCA, CDFILIAL)
                    VALUES
                          (V_NRORCAMENTOGOP, P_NRANOORCAOP, V_IDPERIODOORCA, P_CDVERSORCAOP, C_ORCAMENTO.CDFILIAL);
            ELSE
               V_EXISTEORCA := 'S';
            END IF;   

            BEGIN
               SELECT NVL(IDVERIORCFILI,'NAO_VALIDA')
                 INTO V_IDVERIORCFILI
                 FROM PARASUPRI P, FILIAL F
                WHERE F.CDFILIAL = C_ORCAMENTO.CDFILIAL
                  AND P.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL);
            EXCEPTION
              WHEN NO_DATA_FOUND THEN BEGIN
                V_IDVERIORCFILI := 'NAO_VALIDA';
              END;
            END;

            IF SUBSTR(P_ITEMORCA,1,1) = 'S' THEN
               IF V_IDEXCLUI = 'S' THEN
                  DELETE FROM ITEMORCAGOP
                        WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                          AND IDTIPOORCAOP = 'FATURAMENTO';
               END IF;                          

               OPEN C_SERVICOM;
               FETCH C_SERVICOM INTO V_CDSERVFILI;
               IF C_SERVICOM%ROWCOUNT > 0 THEN
                  LOOP
                     SELECT MAX(NRITEMORCAOP)
                       INTO V_NRITEMORCAGOP
                       FROM ITEMORCAGOP
                      WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                        AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                        AND IDTIPOORCAOP = 'FATURAMENTO'
                        AND CDSERVFILI = V_CDSERVFILI;

                     IF V_NRITEMORCAGOP IS NULL THEN
                        NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                        INSERT INTO ITEMORCAGOP
                                 (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                                  CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                                  DTMESBASEITEM, IDTIPOORCAOP)
                            (SELECT V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                                    '', C_ORCAMENTO.CDFILIAL, '', V_CDSERVFILI, '',
                                    '', 'FATURAMENTO'
                               FROM DUAL
                              MINUS
                             SELECT V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                                    '', C_ORCAMENTO.CDFILIAL, '', V_CDSERVFILI, '',
                                    '', 'FATURAMENTO'
                               FROM ITEMORCAGOP
                              WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                AND IDTIPOORCAOP = 'FATURAMENTO'
                                AND CDSERVFILI = V_CDSERVFILI);
                     END IF;

                     PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'N', 0);
                     
                     FETCH C_SERVICOM INTO V_CDSERVFILI;
                     EXIT WHEN C_SERVICOM%NOTFOUND;
                  END LOOP;
               ELSE
                  SELECT MAX(NRITEMORCAOP)
                    INTO V_NRITEMORCAGOP
                    FROM ITEMORCAGOP
                   WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                     AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                     AND IDTIPOORCAOP = 'FATURAMENTO';

                  IF V_NRITEMORCAGOP IS NULL THEN
                     NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                     INSERT INTO ITEMORCAGOP
                                 (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                                  CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                                  DTMESBASEITEM, IDTIPOORCAOP)
                            (SELECT V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                                    '', C_ORCAMENTO.CDFILIAL, '', '', '',
                                    '', 'FATURAMENTO'
                               FROM DUAL
                              MINUS
                             SELECT V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                                    '', C_ORCAMENTO.CDFILIAL, '', '', '',
                                    '', 'FATURAMENTO'
                               FROM ITEMORCAGOP
                              WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                AND IDTIPOORCAOP = 'FATURAMENTO');
                  END IF;

                  PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'N', 0);
               END IF;
               CLOSE C_SERVICOM;
            END IF;

            IF SUBSTR(P_ITEMORCA,2,1) = 'S' THEN
               IF V_IDEXCLUI = 'S' THEN
                  DELETE FROM ITEMORCAGOP
                        WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                          AND IDTIPOORCAOP = 'IMPOSTO';
               END IF;

               FOR C_IMPOSTO IN (SELECT C_ORCAMENTO.CDFILIAL, PD.CDIMPOSTO
                                   FROM PERCITEMDETALGOP PD INNER JOIN PERCITEMGOP P ON PD.NRPERCITEMGOP = P.NRPERCITEMGOP
                                  WHERE P.NRSEQITEMORCA = 1
                                    AND V_IDUTILPERCFILGOP = 'N'
                                  UNION
                                 SELECT PDF.CDFILIAL, PDF.CDIMPOSTO
                                   FROM PERCITFILGOP PF LEFT JOIN PERCITDETALFILGOP PDF ON PDF.NRPERCITFILGOP = PF.NRPERCITFILGOP
                                  WHERE PDF.CDFILIAL = C_ORCAMENTO.CDFILIAL
                                    AND PF.NRSEQITEMORCA = 1
                                    AND V_IDUTILPERCFILGOP = 'S'
                                  MINUS
                                 SELECT C_ORCAMENTO.CDFILIAL, CDIMPOSTO
                                   FROM ITEMORCAGOP
                                  WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                    AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                    AND IDTIPOORCAOP = 'IMPOSTO') LOOP

                  NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                  INSERT INTO ITEMORCAGOP
                              (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                               CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                               DTMESBASEITEM, IDTIPOORCAOP)
                       VALUES (V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                               '', C_IMPOSTO.CDFILIAL, '', '', C_IMPOSTO.CDIMPOSTO,
                               '', 'IMPOSTO');

                  PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'N', 1);
               END LOOP;
            END IF;

            IF SUBSTR(P_ITEMORCA,3,1) = 'S' THEN
               IF V_IDEXCLUI = 'S' THEN
                  DELETE FROM ITEMORCAGOP
                        WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                          AND IDTIPOORCAOP = 'CONSUMO';
               END IF;

               OPEN C_SERVICOM;
               FETCH C_SERVICOM INTO V_CDSERVFILI;
               IF C_SERVICOM%ROWCOUNT > 0 THEN
                  LOOP
                     SELECT MAX(NRITEMORCAOP)
                       INTO V_NRITEMORCAGOP
                       FROM ITEMORCAGOP
                      WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                        AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                        AND IDTIPOORCAOP = 'CONSUMO'
                        AND CDSERVFILI = V_CDSERVFILI;

                     IF V_NRITEMORCAGOP IS NULL THEN
                        NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                        INSERT INTO ITEMORCAGOP
                                    (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                                     CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                                     DTMESBASEITEM, IDTIPOORCAOP)
                               (SELECT V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                                       '', C_ORCAMENTO.CDFILIAL, '', V_CDSERVFILI, '',
                                       '', 'CONSUMO'
                                  FROM DUAL
                                 MINUS
                                SELECT V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                                       '', C_ORCAMENTO.CDFILIAL, '', V_CDSERVFILI, '',
                                       '', 'CONSUMO'
                                  FROM ITEMORCAGOP
                                 WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                   AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                   AND IDTIPOORCAOP = 'CONSUMO'
                                   AND CDSERVFILI = V_CDSERVFILI);
                     END IF;

                     IF V_IDVERIORCFILI = 'NAO_VALIDA' THEN
                        PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'N', 2);
                     ELSE
                        PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'S', 2);
                     END IF;
                     
                     FETCH C_SERVICOM INTO V_CDSERVFILI;
                     EXIT WHEN C_SERVICOM%NOTFOUND;
                  END LOOP;
               ELSE
                  SELECT MAX(NRITEMORCAOP)
                    INTO V_NRITEMORCAGOP
                    FROM ITEMORCAGOP
                   WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                     AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                     AND IDTIPOORCAOP = 'CONSUMO';

                  IF V_NRITEMORCAGOP IS NULL THEN
                     NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                     INSERT INTO ITEMORCAGOP
                                 (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                                  CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                                  DTMESBASEITEM, IDTIPOORCAOP)
                            (SELECT V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                                    '', C_ORCAMENTO.CDFILIAL, '', '', '',
                                    '', 'CONSUMO'
                               FROM DUAL
                              MINUS
                             SELECT V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                                    '', C_ORCAMENTO.CDFILIAL, '', '', '',
                                    '', 'CONSUMO'
                               FROM ITEMORCAGOP
                              WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                AND IDTIPOORCAOP = 'CONSUMO');
                  END IF;

                  IF V_IDVERIORCFILI = 'NAO_VALIDA' THEN
                     PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'N', 2);
                  ELSE
                     PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'S', 2);
                  END IF;
               END IF;
               CLOSE C_SERVICOM;
            END IF;

            IF SUBSTR(P_ITEMORCA,4,1) = 'S' THEN
               IF V_IDEXCLUI = 'S' THEN
                  DELETE FROM ITEMORCAGOP
                        WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                          AND IDTIPOORCAOP = 'MAOOBRA';
               END IF;

               FOR C_MAOOBRA_FILIAL IN (SELECT C_ORCAMENTO.CDFILIAL, CDDESPADMFAT
                                          FROM DESPADMFATUR
                                         MINUS
                                        SELECT C_ORCAMENTO.CDFILIAL, CDDESPPREV
                                          FROM ITEMORCAGOP
                                         WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                           AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                           AND IDTIPOORCAOP = 'MAOOBRA') LOOP

                  BEGIN
                     SELECT COUNT(CDORCAMENTOMO) AS ORCAFILI
                       INTO V_EXIST_ORCFIL
                       FROM ORCAMENTOMO
                      WHERE CDFILIAL = C_MAOOBRA_FILIAL.CDFILIAL;
                  EXCEPTION
                    WHEN NO_DATA_FOUND THEN
                      V_EXIST_ORCFIL := 0;
                  END;

                  IF V_EXIST_ORCFIL > 0 THEN
                     BEGIN
                        SELECT COUNT(CDORCAMENTOMO)
                          INTO V_EXIST_MOFIL
                          FROM DESPORCAMO
                         WHERE CDFILIAL = C_MAOOBRA_FILIAL.CDFILIAL
                           AND CDDESPADMFAT = C_MAOOBRA_FILIAL.CDDESPADMFAT;
                     EXCEPTION
                       WHEN NO_DATA_FOUND THEN
                         V_EXIST_MOFIL := 0;
                     END;

                     IF V_EXIST_MOFIL > 0 THEN
                        FOR C_ITEMMO IN (SELECT DE.CDDESPADMFAT,DM.CDORCAMENTOMO
                                           FROM DESPORCAMO DM, DESPADMFATUR DE
                                          WHERE DM.CDDESPADMFAT = DE.CDDESPADMFAT
                                            AND DM.CDDESPADMFAT = C_MAOOBRA_FILIAL.CDDESPADMFAT
                                            AND CDFILIAL = C_MAOOBRA_FILIAL.CDFILIAL) LOOP

                           NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                           INSERT INTO ITEMORCAGOP
                                       (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                                        CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                                        DTMESBASEITEM, IDTIPOORCAOP)
                                VALUES (V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', C_ITEMMO.CDORCAMENTOMO,
                                        C_MAOOBRA_FILIAL.CDDESPADMFAT, '', C_MAOOBRA_FILIAL.CDFILIAL, '', '',
                                        '', 'MAOOBRA');

                           PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'S', 3);
                        END LOOP;
                     ELSE
                        BEGIN
                           SELECT COUNT(CDORCAMENTOMO)
                             INTO V_EXIST_MOGER
                             FROM DESPORCAMO
                            WHERE CDFILIAL = 'T'
                              AND CDDESPADMFAT = C_MAOOBRA_FILIAL.CDDESPADMFAT;
                        EXCEPTION
                          WHEN NO_DATA_FOUND THEN
                            V_EXIST_MOGER := 0;
                        END;
                     END IF;
                  END IF;
               END LOOP;

               FOR C_GERALMO IN (SELECT C_ORCAMENTO.CDFILIAL,DE.CDDESPADMFAT,DM.CDORCAMENTOMO
                                   FROM DESPORCAMO DM, DESPADMFATUR DE
                                  WHERE DM.CDDESPADMFAT = DE.CDDESPADMFAT
                                    AND CDFILIAL = 'T'
                                    AND EXISTS (SELECT 1
                                                  FROM ORCAMENTOMO O
                                                 WHERE O.CDFILIAL = 'T'
                                                   AND O.CDORCAMENTOMO = DM.CDORCAMENTOMO)
                                  MINUS
                                 SELECT C_ORCAMENTO.CDFILIAL, CDDESPPREV, CDORCAMENTOMO
                                   FROM ITEMORCAGOP
                                  WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                    AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                    AND IDTIPOORCAOP = 'MAOOBRA') LOOP

                  SELECT COUNT(CDORCAMENTOMO)
                    INTO V_EXIST_MOFIL
                    FROM DESPORCAMO
                   WHERE CDFILIAL     = C_ORCAMENTO.CDFILIAL
                     AND CDDESPADMFAT = C_GERALMO.CDDESPADMFAT;

                  NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                  INSERT INTO ITEMORCAGOP
                              (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                               CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                               DTMESBASEITEM, IDTIPOORCAOP)
                       VALUES (V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', C_GERALMO.CDORCAMENTOMO,
                               C_GERALMO.CDDESPADMFAT, '', 'T', '', '',
                               '', 'MAOOBRA');

                  PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'S', 3);
               END LOOP;
            END IF;

            IF SUBSTR(P_ITEMORCA,5,1) = 'S' THEN
               IF V_IDEXCLUI = 'S' THEN
                  DELETE FROM ITEMORCAGOP
                        WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                          AND IDTIPOORCAOP = 'TAXAADM';
               END IF;

               FOR C_TAXAADM IN (SELECT C_ORCAMENTO.CDFILIAL AS CDFILIAL
                                   FROM DUAL
                                  MINUS
                                 SELECT C_ORCAMENTO.CDFILIAL
                                   FROM ITEMORCAGOP
                                  WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                    AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                    AND IDTIPOORCAOP = 'TAXAADM') LOOP

                  NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                  INSERT INTO ITEMORCAGOP
                              (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                               CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                               DTMESBASEITEM, IDTIPOORCAOP)
                       VALUES (V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                               '', C_TAXAADM.CDFILIAL, '', '', '',
                               '', 'TAXAADM');                                               

                  PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'N', 4);
               END LOOP;
            END IF;

            IF SUBSTR(P_ITEMORCA,6,1) = 'S' THEN
               IF V_IDEXCLUI = 'S' THEN
                  DELETE FROM ITEMORCAGOP
                        WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                          AND IDTIPOORCAOP = 'DESPMATRIZ';
               ELSE
                  FOR C_MATRIZ IN (SELECT NRITEMORCAOP, CDDESPPREV
                                     FROM ITEMORCAGOP
                                    WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                      AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                      AND IDTIPOORCAOP = 'DESPMATRIZ'
                                    GROUP BY NRITEMORCAOP, CDDESPPREV) LOOP
                     PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, C_MATRIZ.NRITEMORCAOP, 'S', 5, C_MATRIZ.CDDESPPREV);
                  END LOOP;
               END IF;

               FOR C_MATRIZ IN (SELECT C_ORCAMENTO.CDFILIAL, CDDESPPREV
                                  FROM DESPPREV
                                 WHERE IDDESPESAMATRIZ = 'S'
                                   AND IDDESPLANENTR <> 'S'
                                   AND IDATIVO = 'S'
                                 MINUS
                                SELECT C_ORCAMENTO.CDFILIAL, CDDESPPREV
                                  FROM ITEMORCAGOP
                                 WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                   AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                   AND IDTIPOORCAOP = 'DESPMATRIZ') LOOP

                  NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                  INSERT INTO ITEMORCAGOP
                              (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                               CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                               DTMESBASEITEM, IDTIPOORCAOP)
                       VALUES (V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                               C_MATRIZ.CDDESPPREV, C_ORCAMENTO.CDFILIAL, '', '', '',
                               '', 'DESPMATRIZ');

                  PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'S', 5, C_MATRIZ.CDDESPPREV);
               END LOOP;
            END IF;

            IF SUBSTR(P_ITEMORCA,7,1) = 'S' THEN
               IF V_IDEXCLUI = 'S' THEN
                  DELETE FROM ITEMORCAGOP
                        WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                          AND IDTIPOORCAOP = 'DESPDIVERSA';
               ELSE
                  FOR C_MATRIZ IN (SELECT NRITEMORCAOP, CDDESPPREV
                                     FROM ITEMORCAGOP
                                    WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                      AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                      AND IDTIPOORCAOP = 'DESPDIVERSA'
                                    GROUP BY NRITEMORCAOP, CDDESPPREV) LOOP
                     PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, C_MATRIZ.NRITEMORCAOP, 'S', 6, C_MATRIZ.CDDESPPREV);
                  END LOOP;
               END IF;

               FOR C_DIVERSAS IN (SELECT C_ORCAMENTO.CDFILIAL, CDDESPPREV
                                    FROM DESPPREV
                                   WHERE IDDESPESAMATRIZ <> 'S'
                                     AND IDDESPLANENTR <> 'S'
                                     AND IDATIVO = 'S'
                                   MINUS
                                  SELECT C_ORCAMENTO.CDFILIAL,CDDESPPREV
                                    FROM ITEMORCAGOP
                                   WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                     AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                     AND IDTIPOORCAOP = 'DESPDIVERSA') LOOP

                  NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                  INSERT INTO ITEMORCAGOP
                              (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                               CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                               DTMESBASEITEM, IDTIPOORCAOP)
                       VALUES (V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                               C_DIVERSAS.CDDESPPREV, C_ORCAMENTO.CDFILIAL, '', '', '',
                               '', 'DESPDIVERSA');                  

                  PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'S', 6, C_DIVERSAS.CDDESPPREV);
               END LOOP;
            END IF;

            IF SUBSTR(P_ITEMORCA,8,1) = 'S' THEN
               IF V_IDEXCLUI = 'S' THEN
                  DELETE FROM ITEMORCAGOP
                        WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                          AND IDTIPOORCAOP = 'DESPENTRADA';
               END IF;

               SELECT IDORCDESPENTROP
                 INTO V_ORCDESPENTROP
                 FROM PARAMORCAOPERAC;

               IF V_ORCDESPENTROP = 'O' THEN
                  OPEN C_DESPENTRADA;
                  FETCH C_DESPENTRADA INTO V_CDTIPOOPER;
                  IF C_DESPENTRADA%ROWCOUNT > 0 THEN
                     LOOP
                        NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                        INSERT INTO ITEMORCAGOP
                                    (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                                     CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                                     DTMESBASEITEM, IDTIPOORCAOP)
                             VALUES (V_NRORCAMENTOGOP, V_NRITEMORCAGOP, 'E', V_CDTIPOOPER, '',
                                     '', C_ORCAMENTO.CDFILIAL, '', '', '',
                                     '', 'DESPENTRADA');

                        PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'N', 7);

                        FETCH C_DESPENTRADA INTO V_CDTIPOOPER;
                        EXIT WHEN C_DESPENTRADA%NOTFOUND;
                     END LOOP;
                  ELSE
                     NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                     INSERT INTO ITEMORCAGOP
                                 (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                                  CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                                  DTMESBASEITEM, IDTIPOORCAOP)
                          VALUES (V_NRORCAMENTOGOP, V_NRITEMORCAGOP, 'E', '', '',
                                  '', C_ORCAMENTO.CDFILIAL, '', '', '',
                                  '', 'DESPENTRADA');

                     PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'N', 7);
                  END IF;
                  CLOSE C_DESPENTRADA;
               ELSE
                  IF V_IDEXCLUI <> 'S' THEN
                     FOR C_MATRIZ IN (SELECT NRITEMORCAOP, CDDESPPREV
                                        FROM ITEMORCAGOP
                                       WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                         AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                         AND IDTIPOORCAOP = 'DESPENTRADA'
                                       GROUP BY NRITEMORCAOP, CDDESPPREV) LOOP
                        PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, C_MATRIZ.NRITEMORCAOP, 'S', 7, C_MATRIZ.CDDESPPREV);
                     END LOOP;
                  END IF;
                  
                  BEGIN
                     SELECT G.IDUTPADRROP, R.IDUTPADRROPFIL
                       INTO V_IDUTPADRROP, V_IDUTPADRROPFIL
                       FROM PARAMGERAL G, PARFILIALROP R
                      WHERE R.CDFILIAL = C_ORCAMENTO.CDFILIAL;
                  EXCEPTION
                    WHEN NO_DATA_FOUND THEN BEGIN
                      V_IDUTPADRROP := 'S';
                      V_IDUTPADRROPFIL := 'S';
                    END;
                  END;

                  FOR C_ENTRADAS_D2 IN (SELECT C_ORCAMENTO.CDFILIAL, DP.CDDESPPREV
                                          FROM DESPPREV DP
                                         WHERE DP.IDDESPESAMATRIZ <> 'S'
                                           AND DP.IDDESPLANENTR = 'S'
                                           AND DP.IDATIVO = 'S'
                                         MINUS
                                        SELECT C_ORCAMENTO.CDFILIAL, CDDESPPREV
                                          FROM ITEMORCAGOP
                                         WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                           AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                           AND IDTIPOORCAOP = 'DESPENTRADA') LOOP

                     NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                     INSERT INTO ITEMORCAGOP
                                 (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                                  CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                                  DTMESBASEITEM, IDTIPOORCAOP)
                          VALUES (V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                                  C_ENTRADAS_D2.CDDESPPREV, C_ORCAMENTO.CDFILIAL, '', '', '',
                                  '', 'DESPENTRADA');

                     PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'S', 7, C_ENTRADAS_D2.CDDESPPREV);
                  END LOOP;
               END IF;
            END IF;

            IF SUBSTR(P_ITEMORCA,9,1) = 'S' THEN
               IF V_IDEXCLUI = 'S' THEN
                  DELETE FROM ITEMORCAGOP
                        WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                          AND IDTIPOORCAOP = 'IMPOSTOARECUPERAR';
               END IF;
               
               OPEN C_IMPOSTOARECUPERAR;
               FETCH C_IMPOSTOARECUPERAR INTO V_CDIMPOSTOREC;
               IF C_IMPOSTOARECUPERAR%ROWCOUNT > 0 THEN
                  LOOP
                     NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                     INSERT INTO ITEMORCAGOP
                                 (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                                  CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                                  DTMESBASEITEM, IDTIPOORCAOP)
                          VALUES (V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                                  '', C_ORCAMENTO.CDFILIAL, '', '', V_CDIMPOSTOREC,
                                  '', 'IMPOSTOARECUPERAR');

                     PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'N', 8);
                     
                     FETCH C_IMPOSTOARECUPERAR INTO V_CDIMPOSTOREC;
                     EXIT WHEN C_IMPOSTOARECUPERAR%NOTFOUND;
                  END LOOP;
               ELSE
                  NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                  INSERT INTO ITEMORCAGOP
                              (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                               CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                               DTMESBASEITEM, IDTIPOORCAOP)
                       VALUES (V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                               '', C_ORCAMENTO.CDFILIAL, '', '', '',
                               '', 'IMPOSTOARECUPERAR');

                  PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'N', 8);
               END IF;
               CLOSE C_IMPOSTOARECUPERAR;
            END IF;

            IF SUBSTR(P_ITEMORCA,10,1) = 'S' THEN
               IF V_IDEXCLUI = 'S' THEN
                  DELETE FROM ITEMORCAGOP
                        WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                          AND IDTIPOORCAOP = 'IMPOSTOSOBLUCRO';
               END IF;

               OPEN C_IMPOSTOSOBLUCRO;
               FETCH C_IMPOSTOSOBLUCRO INTO V_CDIMPOSTOSOB;
               IF C_IMPOSTOSOBLUCRO%ROWCOUNT > 0 THEN
                  LOOP
                     NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                     INSERT INTO ITEMORCAGOP
                                 (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                                  CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                                  DTMESBASEITEM, IDTIPOORCAOP)
                          VALUES (V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                                  '', C_ORCAMENTO.CDFILIAL, '', '', V_CDIMPOSTOSOB,
                                  '', 'IMPOSTOSOBLUCRO');

                     PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'N', 9);
                     
                     FETCH C_IMPOSTOSOBLUCRO INTO V_CDIMPOSTOSOB;
                     EXIT WHEN C_IMPOSTOSOBLUCRO%NOTFOUND;
                  END LOOP;
               ELSE
                  NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                  INSERT INTO ITEMORCAGOP
                              (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                               CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                               DTMESBASEITEM, IDTIPOORCAOP)
                       VALUES (V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                               '', C_ORCAMENTO.CDFILIAL, '', '', '',
                               '', 'IMPOSTOSOBLUCRO');

                  PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'N', 9);
               END IF;
               CLOSE C_IMPOSTOSOBLUCRO;
            END IF;

            IF SUBSTR(P_ITEMORCA,11,1) = 'S' THEN
               IF V_IDEXCLUI = 'S' THEN
                  DELETE FROM ITEMORCAGOP
                        WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                          AND IDTIPOORCAOP = 'INVESTIMENTO';
               END IF;

               FOR C_INVEST IN (SELECT C_ORCAMENTO.CDFILIAL AS CDFILIAL, CDINVEST
                                  FROM INVESTIMENTOS
                                 MINUS
                                SELECT C_ORCAMENTO.CDFILIAL, CDINVEST
                                  FROM ITEMORCAGOP
                                 WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                   AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                   AND IDTIPOORCAOP = 'INVESTIMENTO') LOOP

                  NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                  INSERT INTO ITEMORCAGOP
                              (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                               CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                               DTMESBASEITEM, IDTIPOORCAOP, CDINVEST)
                       VALUES (V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                               '', C_INVEST.CDFILIAL, '', '', '',
                               '', 'INVESTIMENTO', C_INVEST.CDINVEST); 

                  PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'N', 10);
               END LOOP;
            END IF;
            
            IF SUBSTR(P_ITEMORCA,12,1) = 'S' THEN
               IF V_IDEXCLUI = 'S' THEN
                  DELETE FROM ITEMORCAGOP
                        WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                          AND IDTIPOORCAOP = 'DEPRECIACAO';
               END IF;
               FOR C_DEPREC IN (SELECT C_ORCAMENTO.CDFILIAL AS CDFILIAL
                                  FROM DUAL
                                 MINUS
                                SELECT C_ORCAMENTO.CDFILIAL
                                  FROM ITEMORCAGOP
                                 WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP
                                   AND CDFILIAL = C_ORCAMENTO.CDFILIAL
                                   AND IDTIPOORCAOP = 'DEPRECIACAO') LOOP

                  NOVO_CODIGO('ITEMORCAGOP', V_NRITEMORCAGOP);

                  INSERT INTO ITEMORCAGOP
                              (NRORCAMENTOGOP, NRITEMORCAOP, IDENTRSAIDOP, CDTIPOOPER, CDORCAMENTOMO,
                               CDDESPPREV, CDFILIAL, CDFILIALMO, CDSERVFILI, CDIMPOSTO,
                               DTMESBASEITEM, IDTIPOORCAOP)
                       VALUES (V_NRORCAMENTOGOP, V_NRITEMORCAGOP, '', '', '',
                               '', C_DEPREC.CDFILIAL, '', '', '',
                               '', 'DEPRECIACAO');

                  PREPARA_MES(C_ORCAMENTO.CDFILIAL, P_CDVERSORCAOP, V_NRITEMORCAGOP, 'N', 11);
               END LOOP;
            END IF;
         ELSE
            NOVO_CODIGO('LOGPREPARAORCAGOP', V_NRLOGPREPARAORCAGOP);

            IF V_IDAPROVACAOGOP = 'APROVADO' AND NVL(V_OUTRPERIODO,'N') = 'N' THEN
               V_MENSAGEMLOG := 'Orcamento ja aprovado.';
            ELSIF V_STATUSCONC = 'CONCLUIDO' AND NVL(V_OUTRPERIODO,'N') = 'N' THEN
               V_MENSAGEMLOG := 'Orcamento ja concluido.';                        
            ELSIF V_IDAPROVACAOGOP = 'APROVADO' AND NVL(V_OUTRPERIODO,'N') <> 'N' THEN
               V_MENSAGEMLOG := 'Existe um orcamento de periodo '||INITCAP(V_OUTRPERIODO)||' ja aprovado.';
            ELSIF V_STATUSCONC = 'CONCLUIDO' AND NVL(V_OUTRPERIODO,'N') <> 'N' THEN
               V_MENSAGEMLOG := 'Existe um orcamento de periodo '||INITCAP(V_OUTRPERIODO)||' ja concluido.';
            END IF;

            INSERT INTO LOGPREPARAORCAGOP
                        (NRLOGPREPARAORCAGOP, NRANOORCAGOP, IDPERIODOORCA, NRPERIODOORCA, CDFILIAL, DSMOTIVO)
                 VALUES (V_NRLOGPREPARAORCAGOP, P_NRANOORCAOP, V_IDPERIODOORCA, P_CDVERSORCAOP, C_ORCAMENTO.CDFILIAL, V_MENSAGEMLOG);
         END IF;

         IF P_CDOPERLOGADO IS NOT NULL THEN
            -- LOG PREPARACAO ORCAMENTO
            V_OPERADOR := P_CDOPERLOGADO;

            SELECT NMOPERADOR
              INTO V_NMOPERADOR
              FROM OPERADOR
             WHERE CDOPERADOR = V_OPERADOR;

            V_DSSERVERRDM := 'gop00240_preparacao_do_orcamento';
      
            IF V_EXISTEORCA = 'N' THEN
               V_TIPO := '1';
            ELSE
               V_TIPO := '2';
            END IF;      
      
            IF TO_CHAR(C_ORCAMENTO.CDFILIAL) = 'T' OR TO_CHAR(C_ORCAMENTO.CDFILIAL) =  'P' THEN
               V_UNIDADE := 'Unidade: Padro';
            ELSE
               SELECT NMFILIAL
                 INTO V_NMFILIAL
                 FROM FILIAL
                WHERE CDFILIAL = C_ORCAMENTO.CDFILIAL;

                V_UNIDADE := 'Unidade: '|| C_ORCAMENTO.CDFILIAL || ' - ' || V_NMFILIAL;
            END IF;
      
            SELECT NRANOORCAGOP, NRPERIODOORCA
              INTO V_NRANOORCAGOP, V_NRPERIODOORCA
              FROM ORCAMENTOGOP
             WHERE NRORCAMENTOGOP = V_NRORCAMENTOGOP;

            V_DESC := 'Preparao do Oramento' || chr(10) || chr(10) || V_UNIDADE || chr(10);
            V_DESC :=  V_DESC || 'Tipo do Oramento: ' || V_IDPERIODOORCA || chr(10)
                              || 'Ano: ' || V_NRANOORCAGOP || chr(10)
                              || 'Periodo: ' || STRPERIODO(V_NRPERIODOORCA, V_IDPERIODOORCA);
            V_ITENS := ' ';

            IF SUBSTR(P_ITEMORCA,1,1) = 'S' THEN
               V_ITENS := 'Faturamento, ';
            END IF;

            IF SUBSTR(P_ITEMORCA,2,1) = 'S' THEN
               V_ITENS := V_ITENS || 'Imposto, ';
            END IF;

            IF SUBSTR(P_ITEMORCA,3,1) = 'S' THEN
               V_ITENS := V_ITENS || 'Consumo, ';
            END IF;
         
            IF SUBSTR(P_ITEMORCA,4,1) = 'S' THEN
               V_ITENS := V_ITENS || 'Mo de Obra, ';
            END IF;

            IF SUBSTR(P_ITEMORCA,5,1) = 'S' THEN
               V_ITENS := V_ITENS || 'Taxa Administrativa, ';
            END IF;

            IF SUBSTR(P_ITEMORCA,6,1) = 'S' THEN
               V_ITENS := V_ITENS || 'Despesas Matriz, ';
            END IF;

            IF SUBSTR(P_ITEMORCA,7,1) = 'S' THEN
               V_ITENS := V_ITENS || 'Despesas Diversas, ';
            END IF;

            IF SUBSTR(P_ITEMORCA,8,1) = 'S' THEN
               V_ITENS := V_ITENS || 'Despesas Lanamento de Entrada, ';
            END IF;

            IF SUBSTR(P_ITEMORCA,9,1) = 'S' THEN
               V_ITENS := V_ITENS || 'Imposto a Recuperar, ';
            END IF;

            IF SUBSTR(P_ITEMORCA,10,1) = 'S' THEN
               V_ITENS := V_ITENS || 'Imposto sobre o Lucro, ';
            END IF;

            IF SUBSTR(P_ITEMORCA,11,1) = 'S' THEN
               V_ITENS := V_ITENS || 'Investimento, ';
            END IF;

            IF SUBSTR(P_ITEMORCA,12,1) = 'S' THEN
               V_ITENS := V_ITENS || 'Depreciaes, ';
            END IF;

            V_DESC := V_DESC || chr(10) || 'Foi gerado o oramento para os itens: ' || V_ITENS;

            IF V_IDEXCLUI = 'S' THEN
               V_DESC := V_DESC || 'substituindo o oramento do perodo informado, caso existisse.';
            ELSE
               V_DESC := V_DESC || 'respeitando o oramento do perodo informado, caso existisse.';
            END IF;
            V_NMTABELA := 'ORCAMENTOGOP';
            V_ITPESQ := V_NRORCAMENTOGOP;

            INSERT INTO LOGOPERAGES
               (NMTABELA,     DTOPERACAO,   NMRESPOPER,  IDTIPOOPER,   DSOPERACAO,  CDPESQITEM,
                DSRDMUSUOPER, DSENDUSUOPER, DSNMUSUOPER, DSMETUSUOPER, CDORESPOPER, CDFILLOGOPER)
            VALUES
               (V_NMTABELA, SYSDATE, NVL(V_NMOPERADOR,' '), V_TIPO, V_DESC, V_ITPESQ, NVL(V_DSSERVERRDM, ' '),
                ' ', ' ', ' ', NVL(P_CDOPERLOGADO, ' '),
                ' ');
         END IF;
      END LOOP;

      DELETE FROM LISTA WHERE NRLISTA = P_CDOPERADOR;
      DELETE FROM LISTA WHERE NRLISTA = P_CDFILIAL;
      
      COMMIT;
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('PREPARA_ORCAMENTO','6.35.009'); END;
/
